@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -54,6 +54,7 @@ public final class ACLPermission extends MailPermission {
 
     private int canRename;
     private int canStoreSeenFlag;
+    private boolean deputyPermission;
 
     /**
      * Initializes a new {@link ACLPermission}.
@@ -62,6 +63,7 @@ public final class ACLPermission extends MailPermission {
         super();
         canRename = -1;
         canStoreSeenFlag = -1;
+        deputyPermission = false;
     }
 
     @Override
@@ -165,6 +167,20 @@ public final class ACLPermission extends MailPermission {
         this.canStoreSeenFlag = canStoreSeenFlag;
     }
 
+    /**
+     * Sets the deputy permission flag.
+     *
+     * @param deputyPermission The flag to set
+     */
+    public void setDeputyPermission(boolean deputyPermission) {
+        this.deputyPermission = deputyPermission;
+    }
+
+    @Override
+    public boolean isDeputyPermission() {
+        return deputyPermission;
+    }
+
     /*-
      * Full rights: "acdilprsw"
      */
@@ -180,20 +196,19 @@ public final class ACLPermission extends MailPermission {
      * @throws OXException If this permission cannot be mapped to an instance of {@link ACL}
      */
     public ACL getPermissionACL(Entity2ACLArgs args, IMAPConfig imapConfig, IMAPStore imapStore, Context ctx) throws OXException {
-        if (acl != null) {
-            /*
-             * Return caches ACL
-             */
-            return acl;
+        ACL acl = this.acl;
+        if (acl == null) {
+            if (isGroupPermission() && OCLPermission.ALL_GROUPS_AND_USERS != getEntity()) {
+                // Group not supported
+                GroupService groups = Services.getService(GroupService.class);
+                Group group = groups.getGroup(ctx, getEntity());
+                throw Entity2ACLExceptionCode.UNKNOWN_GROUP.create(I(getEntity()), I(ctx.getContextId()), imapConfig.getServer(), group.getDisplayName());
+            }
+            Rights rights = permission2Rights(this, imapConfig);
+            acl = new ACL(Entity2ACL.getInstance(imapStore, imapConfig).getACLName(getEntity(), ctx, args), rights);
+            this.acl = acl;
         }
-        Rights rights = permission2Rights(this, imapConfig);
-        if (isGroupPermission() && OCLPermission.ALL_GROUPS_AND_USERS != getEntity()) {
-            // Group not supported
-            GroupService groups = Services.getService(GroupService.class);
-            Group group = groups.getGroup(ctx, getEntity());
-            throw Entity2ACLExceptionCode.UNKNOWN_GROUP.create(I(getEntity()), I(ctx.getContextId()), imapConfig.getServer(), group.getDisplayName());
-        }
-        return (acl = new ACL(Entity2ACL.getInstance(imapStore, imapConfig).getACLName(getEntity(), ctx, args), rights));
+        return acl;
     }
 
     /**
@@ -214,6 +229,17 @@ public final class ACLPermission extends MailPermission {
         this.acl = acl;
     }
 
+    /**
+     * Parses given rights into this permission object
+     *
+     * @param acl The ACL providing the rights to parse
+     * @param imapConfig The IMAP configuration
+     */
+    public void parseRightsFrom(ACL acl, IMAPConfig imapConfig) {
+        parseRights(acl.getRights(), imapConfig);
+        this.acl = acl;
+    }
+
     /**
      * Parses given rights into this permission object
      *
