@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -122,7 +122,9 @@ public final class AllFetch {
 
             @Override
             public void handleItem(Item item, MailMessage m, IMAPConfig config, org.slf4j.Logger logger) throws OXException {
-                MimeMessageConverter.parseFlags((FLAGS) item, config.getCapabilities().hasAttachmentMarker(), m);
+                int userId = config.getSession().getUserId();
+                int contextId = config.getSession().getContextId();
+                MimeMessageConverter.parseFlags((FLAGS) item, config.getCapabilities().hasAttachmentMarker(), userId, contextId, m);
             }
         }),
         /**
@@ -153,26 +155,13 @@ public final class AllFetch {
                 m.addTo(envelope.to);
                 m.addFrom(envelope.from);
                 // Sender and Reply-To
-                m.addHeader("Sender", addrs2String(envelope.sender));
+                m.addSender(envelope.sender);
                 m.addReplyTo(envelope.replyTo);
                 // In-Reply-To and Message-Id
                 m.addHeader("In-Reply-To", envelope.inReplyTo);
                 m.addHeader("Message-Id", envelope.messageId);
                 m.setSubject(MimeMessageUtility.decodeEnvelopeSubject(envelope.subject), true);
             }
-
-            private String addrs2String(InternetAddress[] addrs) {
-                if (null == addrs || addrs.length == 0) {
-                    return null;
-                }
-                StringBuilder sb = new StringBuilder(addrs.length * 16);
-                sb.append(addrs[0].toString());
-                for (int i = 1; i < addrs.length; i++) {
-                    sb.append(", ").append(addrs[i].toString());
-                }
-                return sb.toString();
-            }
-
         });
 
         private final String item;
