@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -22,6 +22,7 @@
 package com.openexchange.imap;
 
 import static com.openexchange.java.Autoboxing.I;
+import static com.openexchange.mailaccount.MailAccounts.isSecondaryAccount;
 import static com.sun.mail.iap.ResponseCode.AUTHENTICATIONFAILED;
 import java.io.IOException;
 import java.security.MessageDigest;
@@ -51,6 +52,7 @@ import com.openexchange.config.ConfigurationService;
 import com.openexchange.exception.OXException;
 import com.openexchange.imap.acl.ACLExtension;
 import com.openexchange.imap.acl.ACLExtensionInit;
+import com.openexchange.imap.cache.FileNameSearchSupportedCache;
 import com.openexchange.imap.cache.ListLsubCache;
 import com.openexchange.imap.cache.ListLsubEntry;
 import com.openexchange.imap.cache.MBoxEnabledCache;
@@ -93,6 +95,9 @@ import com.openexchange.mail.mime.MimeMailExceptionCode;
 import com.openexchange.mail.mime.MimeSessionPropertyNames;
 import com.openexchange.mailaccount.MailAccount;
 import com.openexchange.mailaccount.MailAccountStorageService;
+import com.openexchange.mailaccount.MailAccounts;
+import com.openexchange.mailaccount.utils.MailAccountUtils;
+import com.openexchange.net.HostList;
 import com.openexchange.net.ssl.SSLSocketFactoryProvider;
 import com.openexchange.net.ssl.config.SSLConfigurationService;
 import com.openexchange.net.ssl.exception.SSLExceptionCode;
@@ -441,13 +446,20 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
             final IMAPStore imapStore = this.imapStore;
             if (imapStore != null) {
                 if (useIMAPStoreCache()) {
-                    final IMAPStoreCache imapStoreCache = IMAPStoreCache.getInstance();
+                    IMAPStoreCache imapStoreCache = IMAPStoreCache.getInstance();
                     if (null == imapStoreCache) {
                         closeSafely(imapStore);
                     } else if (imapStore.isConnectedUnsafe()) {
-                        imapStoreCache.returnIMAPStore(imapStore, accountId, server, port, login, session);
+                        if (imapStore.hasAuthenticatedConnections()) {
+                            imapStoreCache.returnIMAPStore(imapStore, accountId, server, port, login, session);
+                        } else {
+                            // Holds no authenticated connections anymore. Thus close it since new connection needs to be established anyway.
+                            imapStoreCache.signalClosed(accountId, server, port, login, session);
+                            closeSafely(imapStore);
+                        }
                     } else {
                         // Not null AND not connected...?
+                        imapStoreCache.signalClosed(accountId, server, port, login, session);
                         closeSafely(imapStore);
                     }
                 } else {
@@ -511,7 +523,7 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
             case OAUTH:
                 try {
                     IMAPConfig imapConfig = getIMAPConfig();
-                    boolean isPrimary = (imapConfig.getAccountId() == MailAccount.DEFAULT_ID);
+                    boolean isPrimary = (imapConfig.getAccountId() == MailAccount.DEFAULT_ID || MailAccounts.isSecondaryAccount(accountId, session));
                     return IMAPCapabilityAndGreetingCache.getCapabilities(new HostAndPort(IDNA.toASCII(imapConfig.getServer()), imapConfig.getPort()), imapConfig.isSecure(), imapConfig.getIMAPProperties(), isPrimary).containsKey("AUTH=XOAUTH2");
                 } catch (IOException e) {
                     throw MailExceptionCode.IO_ERROR.create(e, e.getMessage());
@@ -519,7 +531,7 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
             case OAUTHBEARER:
                 try {
                     IMAPConfig imapConfig = getIMAPConfig();
-                    boolean isPrimary = (imapConfig.getAccountId() == MailAccount.DEFAULT_ID);
+                    boolean isPrimary = (imapConfig.getAccountId() == MailAccount.DEFAULT_ID || MailAccounts.isSecondaryAccount(accountId, session));
                     return IMAPCapabilityAndGreetingCache.getCapabilities(new HostAndPort(IDNA.toASCII(imapConfig.getServer()), imapConfig.getPort()), imapConfig.isSecure(), imapConfig.getIMAPProperties(), isPrimary).containsKey("AUTH=OAUTHBEARER");
                 } catch (IOException e) {
                     throw MailExceptionCode.IO_ERROR.create(e, e.getMessage());
@@ -546,7 +558,7 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
              */
             final IMAPFolder imapFolder = (IMAPFolder) imapStore.getFolder(fullname);
             final IMAPConfig imapConfig = getIMAPConfig();
-            final ListLsubEntry listEntry = ListLsubCache.getCachedLISTEntry(fullname, accountId, imapFolder, session, imapConfig.getIMAPProperties().isIgnoreSubscription());
+            final ListLsubEntry listEntry = ListLsubCache.getCachedLISTEntry(fullname, accountId, imapFolder, session, imapConfig.getIMAPProperties());
             final boolean exists = "INBOX".equals(fullname) || (listEntry.exists());
             if (!exists) {
                 throw IMAPException.create(IMAPException.Code.FOLDER_NOT_FOUND, imapConfig, session, fullname);
@@ -620,10 +632,16 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
                 imapSession.setDebug(true);
                 imapSession.setDebugOut(System.err);
             } else if (PropUtil.getBooleanProperty(imapSession.getProperties(), "mail.imap.debugLog.enabled", false)) {
-                /*
-                 * Debug logging is enabled for this IMAP session
-                 */
-                establishDebugLogger(imapSession, server, port);
+                Object serverPattern = imapSession.getProperties().get("mail.imap.debugLog.serverPattern");
+                if (serverPattern == null) {
+                    serverPattern = imapSession.getProperties().getProperty("mail.imap.debugLog.serverPattern");
+                }
+                if (serverPattern == null || server.indexOf(serverPattern.toString()) >= 0) {
+                    /*
+                     * Debug logging is enabled for this IMAP session
+                     */
+                    establishDebugLogger(imapSession, server, port);
+                }
             }
             IMAPStore imapStore = null;
             try {
@@ -652,12 +670,15 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
                 throw MimeMailException.handleMessagingException(e, config, session);
             } catch (MessagingException e) {
                 if (MimeMailException.isSSLHandshakeException(e)) {
-                    List<Object> displayArgs = new ArrayList<>(2);
-                    displayArgs.add(SSLExceptionCode.extractArgument(e, "fingerprint"));
-                    displayArgs.add(config.getServer());
-                    OXException oxe = SSLExceptionCode.UNTRUSTED_CERTIFICATE.create(e.getCause(), displayArgs.toArray(new Object[] {}));
-                    warnings.add(oxe);
-                    throw oxe;
+                    String fingerPrint = SSLExceptionCode.extractArgument(e, "fingerprint");
+                    if (Strings.isNotEmpty(fingerPrint)) {
+                        List<Object> displayArgs = new ArrayList<>(2);
+                        displayArgs.add(fingerPrint);
+                        displayArgs.add(server);
+                        OXException oxe = SSLExceptionCode.UNTRUSTED_CERTIFICATE.create(e.getCause(), displayArgs.toArray(new Object[] {}));
+                        warnings.add(oxe);
+                        throw oxe;
+                    }
                 }
 
                 Exception cause = e.getNextException();
@@ -785,10 +806,16 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
                 imapSession.setDebug(true);
                 imapSession.setDebugOut(System.out);
             } else if (PropUtil.getBooleanProperty(imapSession.getProperties(), "mail.imap.debugLog.enabled", false)) {
-                /*
-                 * Debug logging is enabled for this IMAP session
-                 */
-                establishDebugLogger(imapSession, server, port);
+                Object serverPattern = imapSession.getProperties().get("mail.imap.debugLog.serverPattern");
+                if (serverPattern == null) {
+                    serverPattern = imapSession.getProperties().getProperty("mail.imap.debugLog.serverPattern");
+                }
+                if (serverPattern == null || server.indexOf(serverPattern.toString()) >= 0) {
+                    /*
+                     * Debug logging is enabled for this IMAP session
+                     */
+                    establishDebugLogger(imapSession, server, port);
+                }
             }
             /*
              * Check for already failed authentication
@@ -848,10 +875,13 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
                     }
                 }
                 if (MimeMailException.isSSLHandshakeException(e)) {
-                    List<Object> displayArgs = new ArrayList<>(2);
-                    displayArgs.add(SSLExceptionCode.extractArgument(e, "fingerprint"));
-                    displayArgs.add(server);
-                    throw SSLExceptionCode.UNTRUSTED_CERTIFICATE.create(e.getCause(), displayArgs.toArray(new Object[] {}));
+                    String fingerPrint = SSLExceptionCode.extractArgument(e, "fingerprint");
+                    if (Strings.isNotEmpty(fingerPrint)) {
+                        List<Object> displayArgs = new ArrayList<>(2);
+                        displayArgs.add(fingerPrint);
+                        displayArgs.add(server);
+                        throw SSLExceptionCode.UNTRUSTED_CERTIFICATE.create(e.getCause(), displayArgs.toArray(new Object[] {}));
+                    }
                 }
                 {
                     Exception next = e.getNextException();
@@ -907,6 +937,9 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
                 }
             }
         } catch (MessagingException e) {
+            if (e.getNextException() instanceof ConnectQuotaExceededException) {
+                throw IMAPException.create(IMAPException.Code.CONNECTION_UNAVAILABLE, e.getNextException(), config.getServer(), config.getLogin());
+            }
             throw MimeMailException.handleMessagingException(e, config, session);
         }
     }
@@ -951,10 +984,11 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
         /*
          * Propagate client IP address
          */
+        Properties properties = imapSession.getProperties();
         if (clientIp != null) {
-            imapSession.getProperties().put("mail.imap.propagate.clientipaddress", clientIp);
+            properties.put("mail.imap.propagate.clientipaddress", clientIp);
         }
-        imapSession.getProperties().put("mail.imap.failOnNOFetch", "true");
+        properties.put("mail.imap.failOnNOFetch", "true");
 
         /*
          * Set log properties
@@ -970,10 +1004,9 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
          */
         if (useIMAPStoreCache()) {
             /*
-             * Possible connect limitation
+             * Possible connection limitation
              */
             if (maxCount > 0) {
-                final Properties properties = imapSession.getProperties();
                 properties.put("mail.imap.maxNumAuthenticated", Integer.toString(maxCount));
                 properties.put("mail.imap.authAwait", "true");
                 properties.put("mail.imap.accountId", Integer.toString(accountId));
@@ -986,12 +1019,12 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
             return borrowedIMAPStore;
         }
         /*
-         * Possible connect limitation
+         * Possible connection limitation
          */
         if (maxCount > 0) {
-            imapSession.getProperties().put("mail.imap.maxNumAuthenticated", Integer.toString(maxCount));
-            imapSession.getProperties().put("mail.imap.authAwait", "true");
-            imapSession.getProperties().put("mail.imap.accountId", Integer.toString(accountId));
+            properties.put("mail.imap.maxNumAuthenticated", Integer.toString(maxCount));
+            properties.put("mail.imap.authAwait", "true");
+            properties.put("mail.imap.accountId", Integer.toString(accountId));
         }
         /*
          * Retry loop...
@@ -1021,21 +1054,24 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
         /*
          * Establish a new one...
          */
-        IMAPStore imapStore = (IMAPStore) imapSession.getStore(PROTOCOL);
         boolean isPrimary = false;
-        if (MailAccount.DEFAULT_ID == accountId) {
-            isPrimary = true;
-            boolean failedToSetClientParameters = true; // Pessimistic
-            try {
-                IMAPClientParameters.setDefaultClientParameters(imapStore, session);
-                failedToSetClientParameters = false;
-            } catch (OXException e) {
-                throw new MessagingException(e.getMessage(), e);
-            } finally {
-                if (failedToSetClientParameters) {
-                    Streams.close(imapStore);
+        IMAPStore imapStore = (IMAPStore) imapSession.getStore(PROTOCOL);
+        try {
+            if (MailAccount.DEFAULT_ID == accountId || isSecondaryAccount(accountId, session)) {
+                isPrimary = true;
+                boolean failedToSetClientParameters = true; // Pessimistic
+                try {
+                    String remotePort = LogProperties.get(LogProperties.Name.GRIZZLY_REMOTE_PORT);
+                    IMAPClientParameters.setDefaultClientParameters(imapStore, session, Strings.parsePositiveInt(remotePort));
+                    failedToSetClientParameters = false;
+                } finally {
+                    if (failedToSetClientParameters) {
+                        Streams.close(imapStore);
+                    }
                 }
             }
+        } catch (OXException e) {
+            throw new MessagingException(e.getMessage(), e);
         }
         /*
          * ... and connect it
@@ -1229,6 +1265,7 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
         IMAPCapabilityAndGreetingCache.init();
         MBoxEnabledCache.init();
         RootSubfoldersEnabledCache.init();
+        FileNameSearchSupportedCache.init();
         ACLExtensionInit.getInstance().start();
         Entity2ACLInit.getInstance().start();
 
@@ -1289,6 +1326,7 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
         IMAPCapabilityAndGreetingCache.tearDown();
         MBoxEnabledCache.tearDown();
         RootSubfoldersEnabledCache.tearDown();
+        FileNameSearchSupportedCache.tearDown();
         IMAPSessionProperties.resetDefaultSessionProperties();
         dropMaps();
     }
@@ -1377,6 +1415,10 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
          */
         if (config.getIMAPProperties().isDebugLogEnabled()) {
             imapProps.put("mail.imap.debugLog.enabled", "true");
+            String serverPattern = config.getIMAPProperties().getDebugServerPattern();
+            if (Strings.isNotEmpty(serverPattern)) {
+                imapProps.put("mail.imap.debugLog.serverPattern", serverPattern);
+            }
         }
         /*
          * Greeting listener (for primary IMAP)
@@ -1390,7 +1432,7 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
         /*
          * Allow round-robin address election for primary IMAP account
          */
-        if (config.getAccountId() == MailAccount.DEFAULT_ID) {
+        if (config.getAccountId() == MailAccount.DEFAULT_ID || MailAccounts.isSecondaryAccount(config.getAccountId(), userId, contextId)) {
             imapProps.put("mail.imap.primary", "true");
             boolean useMultipleAddresses = IMAPProperties.getInstance().isUseMultipleAddresses(userId, contextId);
             if (useMultipleAddresses) {
@@ -1411,6 +1453,14 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
                     imapProps.put("mail.imap.multiAddress.maxRetries", Integer.toString(maxRetries));
                 }
             }
+        } else {
+            /*
+             * Blocked hosts
+             */
+            HostList blockedHosts = MailAccountUtils.getBlockedHosts();
+            if (!blockedHosts.isEmpty()) {
+                imapProps.put("mail.imap.blockedHosts", blockedHosts);
+            }
         }
         /*
          * Enable XOAUTH2/OAUTHBEARER (if appropriate)
@@ -1528,7 +1578,7 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
          * Create new IMAP session from initialized properties
          */
         final javax.mail.Session imapSession = javax.mail.Session.getInstance(imapProps, null);
-        imapSession.addProvider(new Provider(Provider.Type.STORE, "imap", storeClass.getName(), "OX Software GmbH", getVersion()));
+        imapSession.addProvider(new Provider(Provider.Type.STORE, "imap", storeClass.getName(), "Open-Xchange GmbH", getVersion()));
         return imapSession;
     }
 
