@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -21,6 +21,7 @@
 
 package com.openexchange.imap;
 
+import com.openexchange.mail.PreviewMode;
 import com.openexchange.mail.api.MailCapabilities;
 
 /**
@@ -99,23 +100,33 @@ public class IMAPCapabilities extends MailCapabilities {
     /**
      * TEXT PREVIEW: <code>"SNIPPET=FUZZY"</code>
      */
-    public static final String CAP_TEXT_PREVIEW = "SNIPPET=FUZZY";
+    public static final String CAP_TEXT_PREVIEW = PreviewMode.SNIPPET_FUZZY.getCapabilityName();
 
     /**
      * TEXT PREVIEW: <code>"PREVIEW=FUZZY"</code>
      */
-    public static final String CAP_TEXT_PREVIEW_NEW = "PREVIEW=FUZZY";
+    public static final String CAP_TEXT_PREVIEW_NEW = PreviewMode.PREVIEW_FUZZY.getCapabilityName();
+
+    /**
+     * RFC8970-conform TEXT PREVIEW: <code>"PREVIEW"</code>
+     */
+    public static final String CAP_TEXT_PREVIEW_RFC8970 = PreviewMode.PREVIEW_RFC8970.getCapabilityName();
 
     /**
      * CONDSTORE support according to <a href="https://tools.ietf.org/html/rfc7162">https://tools.ietf.org/html/rfc7162</a>
      */
     public static final String CAP_CONDSTORE = "CONDSTORE";
-    
+
     /**
      * Filters applicable to existing messages: <code>"FILTER=SIEVE"</code>
      */
     public static final String CAP_FILTER_SIEVE = "FILTER=SIEVE";
 
+    /**
+     * Allows setting meta-data: <code>"METADATA=SIEVE"</code>
+     */
+    public static final String CAP_METADATA = "METADATA";
+
     /*-
      * IMAP bit constants
      */
@@ -159,6 +170,8 @@ public class IMAPCapabilities extends MailCapabilities {
     private boolean hasMailFilterApplication;
     private boolean hasPublicFolders;
     private boolean hasSharedFolders;
+    private boolean hasMetadata;
+    private Boolean hasCustomFlag;
 
     /**
      * Initializes a new {@link IMAPCapabilities}
@@ -334,6 +347,40 @@ public class IMAPCapabilities extends MailCapabilities {
         return hasTextPreview;
     }
 
+    @Override
+    public boolean hasMailStructure() {
+        // Through IMAP BODYSTRUCTURE
+        return true;
+    }
+
+    /**
+     * Sets whether custom flag is supported
+     *
+     * @param hasGroupEntities <code>true</code> to signal support for custom flag; otherwise <code>false</code>
+     */
+    public void setCustomFlag(boolean hasCustomFlag) {
+        this.hasCustomFlag = (hasCustomFlag ? Boolean.TRUE : Boolean.FALSE);
+    }
+
+    @Override
+    public boolean hasCustomFlag() {
+        return hasCustomFlag == null ? super.hasCustomFlag() : hasCustomFlag.booleanValue();
+    }
+
+    /**
+     * Sets whether meta-data is supported
+     *
+     * @param hasMetadata <code>true</code> to signal support for meta-data; otherwise <code>false</code>
+     */
+    public void setMetadata(boolean hasMetadata) {
+        this.hasMetadata = hasMetadata;
+    }
+
+    @Override
+    public boolean hasMetadata() {
+        return hasMetadata;
+    }
+
     @Override
     public boolean hasAttachmentMarker() {
         return hasAttachmentSearch;
