@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -47,6 +47,10 @@ public enum IMAPClientParameters {
      * The parameter for the client's originating IP address.
      */
     ORIGINATING_IP("x-originating-ip"),
+    /**
+     * The parameter for the client's originating port number.
+     */
+    ORIGINATING_PORT("x-originating-port"),
     /**
      * The parameter for the client's session identifier.
      */
@@ -209,9 +213,10 @@ public enum IMAPClientParameters {
      *
      * @param imapStore The IMAP store to connect to
      * @param session The associated Groupware session
-     * @throws OXException
+     * @param remotePort The remote IMAP client's TCP port number or less than/equal to <code>0</code> (zero) to ignore
+     * @throws OXException If client parameters cannot be applied
      */
-    public static void setDefaultClientParameters(IMAPStore imapStore, Session session) throws OXException {
+    public static void setDefaultClientParameters(IMAPStore imapStore, Session session, int remotePort) throws OXException {
         // Set generator
         imapStore.setExternalIdGenerator(new Generator(session));
 
@@ -221,6 +226,9 @@ public enum IMAPClientParameters {
         clientParams.put(IMAPClientParameters.ORIGINATING_IP.getParamName(), Strings.isEmpty(localIp) ? LOCAL_HOST : localIp);
         clientParams.put(IMAPClientParameters.NAME.getParamName(), "Open-Xchange");
         clientParams.put(IMAPClientParameters.VERSION.getParamName(), Services.getServiceSafe(VersionService.class).getVersionString());
+        if (remotePort > 0) {
+            clientParams.put(IMAPClientParameters.ORIGINATING_PORT.getParamName(), Integer.toString(remotePort));
+        }
         imapStore.setClientParameters(clientParams);
     }
 
