@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -49,6 +49,7 @@ import com.openexchange.imap.command.MailMessageFillerIMAPCommand;
 import com.openexchange.imap.command.MessageFetchIMAPCommand;
 import com.openexchange.imap.command.SimpleFetchIMAPCommand;
 import com.openexchange.imap.config.IMAPConfig;
+import com.openexchange.imap.config.IMAPProperties;
 import com.openexchange.imap.config.IMAPReloadable;
 import com.openexchange.imap.search.IMAPSearch;
 import com.openexchange.imap.services.Services;
@@ -68,6 +69,7 @@ import com.openexchange.mail.MailField;
 import com.openexchange.mail.MailFields;
 import com.openexchange.mail.MailSortField;
 import com.openexchange.mail.OrderDirection;
+import com.openexchange.mail.PreviewMode;
 import com.openexchange.mail.api.IMailFolderStorage;
 import com.openexchange.mail.api.IMailMessageStorage;
 import com.openexchange.mail.api.MailAccess;
@@ -79,6 +81,7 @@ import com.openexchange.mail.search.SearchTerm;
 import com.openexchange.mail.utils.MailMessageComparator;
 import com.openexchange.mail.utils.MailMessageComparatorFactory;
 import com.openexchange.mailaccount.MailAccount;
+import com.openexchange.session.IUserAndContext;
 import com.openexchange.session.Session;
 import com.openexchange.threadpool.AbstractTask;
 import com.openexchange.threadpool.ThreadPools;
@@ -366,14 +369,15 @@ public final class IMAPConversationWorker {
             long uidNext = imapMessageStorage.getImapFolder().getUIDNext();
             long highestModSeq = supportsCondStore ? imapMessageStorage.getImapFolder().getHighestModSeq() : -1L;
             boolean examineHasAttachmentUserFlags = imapMessageStorage.examineHasAttachmentUserFlags;
-            boolean previewSupported = imapMessageStorage.previewSupported;
+            IUserAndContext userAndContext = imapMessageStorage.session;
+            PreviewMode previewMode = imapMessageStorage.previewMode;
             FetchProfile fp;
             if (byEnvelope) {
-                fp = Conversations.getFetchProfileConversationByEnvelope(examineHasAttachmentUserFlags, previewSupported, null == sortField ? MailField.RECEIVED_DATE : MailField.toField(sortField.getListField()));
+                fp = Conversations.getFetchProfileConversationByEnvelope(examineHasAttachmentUserFlags, previewMode, null == sortField ? MailField.RECEIVED_DATE : MailField.toField(sortField.getListField()));
             } else {
-                fp = Conversations.getFetchProfileConversationByHeaders(examineHasAttachmentUserFlags, previewSupported, null == sortField ? MailField.RECEIVED_DATE : MailField.toField(sortField.getListField()));
+                fp = Conversations.getFetchProfileConversationByHeaders(examineHasAttachmentUserFlags, previewMode, null == sortField ? MailField.RECEIVED_DATE : MailField.toField(sortField.getListField()));
             }
-            conversations = Conversations.conversationsFor(imapMessageStorage.getImapFolder(), lookAhead, order, fp, imapMessageStorage.getImapServerInfo(), byEnvelope, examineHasAttachmentUserFlags, previewSupported);
+            conversations = Conversations.conversationsFor(imapMessageStorage.getImapFolder(), lookAhead, order, fp, imapMessageStorage.getImapServerInfo(), byEnvelope, examineHasAttachmentUserFlags, userAndContext, previewMode);
             if (conversations.isEmpty()) {
                 return Collections.emptyList();
             }
@@ -388,7 +392,7 @@ public final class IMAPConversationWorker {
                 sentUidNext = imapMessageStorage.getImapFolder().getUIDNext();
                 sentHighestModSeq = supportsCondStore ? imapMessageStorage.getImapFolder().getHighestModSeq() : -1L;
                 // Get sent messages
-                List<MailMessage> sentMessages = Conversations.messagesFor(imapMessageStorage.getImapFolder(), lookAhead, order, fp, imapMessageStorage.getImapServerInfo(), byEnvelope, examineHasAttachmentUserFlags, previewSupported);
+                List<MailMessage> sentMessages = Conversations.messagesFor(imapMessageStorage.getImapFolder(), lookAhead, order, fp, imapMessageStorage.getImapServerInfo(), byEnvelope, examineHasAttachmentUserFlags, userAndContext, previewMode);
                 if (false == sentMessages.isEmpty()) {
                     // Filter messages already contained in conversations
                     {
@@ -627,9 +631,10 @@ public final class IMAPConversationWorker {
         imapFolder.open(IMAPFolder.READ_ONLY);
         try {
             boolean examineHasAttachmentUserFlags = mailConfig.getCapabilities().hasAttachmentMarker();
-            boolean previewSupported = mailConfig.asMap().containsKey(IMAPCapabilities.CAP_TEXT_PREVIEW_NEW);
+            IUserAndContext userAndContext = mailConfig.getSession();
+            PreviewMode previewMode = IMAPProperties.previewModeFor(mailConfig);
             if (mergeWithSent) {
-                FetchProfile fetchProfile = IMAPMessageStorage.checkFetchProfile(getFetchProfile(usedFields.toArray(), headerNames, null, null, true, examineHasAttachmentUserFlags, previewSupported));
+                FetchProfile fetchProfile = IMAPMessageStorage.checkFetchProfile(getFetchProfile(usedFields.toArray(), headerNames, null, null, true, examineHasAttachmentUserFlags && usedFields.containsAny(MailField.ATTACHMENT, MailField.CONTENT_TYPE), previewMode));
                 List<MailMessage> msgs = new LinkedList<>();
                 List<MailMessage> sentmsgs = new LinkedList<>();
                 for (List<MailMessage> conversation : list) {
@@ -641,20 +646,20 @@ public final class IMAPConversationWorker {
                         }
                     }
                 }
-                new MailMessageFillerIMAPCommand(msgs, isRev1, fetchProfile, imapServerInfo, examineHasAttachmentUserFlags, previewSupported, imapFolder).doCommand();
+                new MailMessageFillerIMAPCommand(msgs, isRev1, fetchProfile, imapServerInfo, examineHasAttachmentUserFlags, userAndContext, previewMode, imapFolder).doCommand();
                 if (!sentmsgs.isEmpty()) {
                     // Switch folder
                     imapFolder.close(false);
                     imapFolder = (IMAPFolder) imapStore.getFolder(sentFullName);
                     imapFolder.open(IMAPFolder.READ_ONLY);
-                    new MailMessageFillerIMAPCommand(sentmsgs, isRev1, fetchProfile, imapServerInfo, examineHasAttachmentUserFlags, previewSupported, imapFolder).doCommand();
+                    new MailMessageFillerIMAPCommand(sentmsgs, isRev1, fetchProfile, imapServerInfo, examineHasAttachmentUserFlags, userAndContext, previewMode, imapFolder).doCommand();
                 }
             } else {
                 List<MailMessage> msgs = new LinkedList<>();
                 for (List<MailMessage> conversation : list) {
                     msgs.addAll(conversation);
                 }
-                new MailMessageFillerIMAPCommand(msgs, isRev1, getFetchProfile(usedFields.toArray(), headerNames, null, null, true, examineHasAttachmentUserFlags, previewSupported), imapServerInfo, examineHasAttachmentUserFlags, previewSupported, imapFolder).doCommand();
+                new MailMessageFillerIMAPCommand(msgs, isRev1, getFetchProfile(usedFields.toArray(), headerNames, null, null, true, examineHasAttachmentUserFlags && usedFields.containsAny(MailField.ATTACHMENT, MailField.CONTENT_TYPE), previewMode), imapServerInfo, examineHasAttachmentUserFlags, userAndContext, previewMode, imapFolder).doCommand();
             }
             /*
              * Apply account identifier
@@ -669,9 +674,10 @@ public final class IMAPConversationWorker {
         // Fill messages
         List<List<MailMessage>> l1st = list;
         boolean examineHasAttachmentUserFlags = imapMessageStorage.examineHasAttachmentUserFlags;
-        boolean previewSupported = imapMessageStorage.previewSupported;
+        IUserAndContext userAndContext = imapMessageStorage.session;
+        PreviewMode previewMode = imapMessageStorage.previewMode;
         if (mergeWithSent) {
-            FetchProfile fetchProfile = IMAPMessageStorage.checkFetchProfile(getFetchProfile(usedFields.toArray(), headerNames, null, null, true, examineHasAttachmentUserFlags, previewSupported));
+            FetchProfile fetchProfile = IMAPMessageStorage.checkFetchProfile(getFetchProfile(usedFields.toArray(), headerNames, null, null, true, examineHasAttachmentUserFlags && usedFields.containsAny(MailField.ATTACHMENT, MailField.CONTENT_TYPE), previewMode));
             List<MailMessage> msgs = new LinkedList<>();
             List<MailMessage> sentmsgs = new LinkedList<>();
             for (List<MailMessage> conversation : l1st) {
@@ -683,11 +689,11 @@ public final class IMAPConversationWorker {
                     }
                 }
             }
-            new MailMessageFillerIMAPCommand(msgs, isRev1, fetchProfile, imapMessageStorage.getImapServerInfo(), examineHasAttachmentUserFlags, previewSupported, imapMessageStorage.getImapFolder()).doCommand();
+            new MailMessageFillerIMAPCommand(msgs, isRev1, fetchProfile, imapMessageStorage.getImapServerInfo(), examineHasAttachmentUserFlags, userAndContext, previewMode, imapMessageStorage.getImapFolder()).doCommand();
             if (!sentmsgs.isEmpty()) {
                 // Switch folder
                 imapMessageStorage.openReadOnly(sentFullName);
-                new MailMessageFillerIMAPCommand(sentmsgs, isRev1, fetchProfile, imapMessageStorage.getImapServerInfo(), examineHasAttachmentUserFlags, previewSupported, imapMessageStorage.getImapFolder()).doCommand();
+                new MailMessageFillerIMAPCommand(sentmsgs, isRev1, fetchProfile, imapMessageStorage.getImapServerInfo(), examineHasAttachmentUserFlags, userAndContext, previewMode, imapMessageStorage.getImapFolder()).doCommand();
                 // Switch back folder
                 imapMessageStorage.openReadOnly(fullName);
             }
@@ -707,7 +713,7 @@ public final class IMAPConversationWorker {
                 for (List<MailMessage> conversation : l1st) {
                     msgs.addAll(conversation);
                 }
-                new MailMessageFillerIMAPCommand(msgs, isRev1, getFetchProfile(usedFields.toArray(), headerNames, null, null, true, examineHasAttachmentUserFlags, previewSupported), imapMessageStorage.getImapServerInfo(), examineHasAttachmentUserFlags, previewSupported, imapMessageStorage.getImapFolder()).doCommand();
+                new MailMessageFillerIMAPCommand(msgs, isRev1, getFetchProfile(usedFields.toArray(), headerNames, null, null, true, examineHasAttachmentUserFlags && usedFields.containsAny(MailField.ATTACHMENT, MailField.CONTENT_TYPE), previewMode), imapMessageStorage.getImapServerInfo(), examineHasAttachmentUserFlags, userAndContext, previewMode, imapMessageStorage.getImapFolder()).doCommand();
             }
         }
         /*
@@ -727,25 +733,26 @@ public final class IMAPConversationWorker {
         if (body && mergeWithSent) {
             throw MailExceptionCode.ILLEGAL_ARGUMENT.create();
         }
-        final boolean byEnvelope = false;
-        final boolean isRev1 = imapMessageStorage.getImapConfig().getImapCapabilities().hasIMAP4rev1();
-        final boolean examineHasAttachmentUserFlags = imapMessageStorage.examineHasAttachmentUserFlags;
-        final boolean previewSupported = imapMessageStorage.previewSupported;
+        boolean byEnvelope = false;
+        boolean isRev1 = imapMessageStorage.getImapConfig().getImapCapabilities().hasIMAP4rev1();
+        boolean examineHasAttachmentUserFlags = imapMessageStorage.examineHasAttachmentUserFlags;
+        IUserAndContext userAndContext = imapMessageStorage.session;
+        PreviewMode previewMode = imapMessageStorage.previewMode;
 
         List<List<MailMessage>> list;
         if (mergeWithSent) {
             FetchProfile fp;
             if (byEnvelope) {
-                fp = Conversations.getFetchProfileConversationByEnvelope(examineHasAttachmentUserFlags, previewSupported, null == sortField ? MailField.RECEIVED_DATE : MailField.toField(sortField.getListField()));
+                fp = Conversations.getFetchProfileConversationByEnvelope(examineHasAttachmentUserFlags, previewMode, null == sortField ? MailField.RECEIVED_DATE : MailField.toField(sortField.getListField()));
             } else {
-                fp = Conversations.getFetchProfileConversationByHeaders(examineHasAttachmentUserFlags, previewSupported, null == sortField ? MailField.RECEIVED_DATE : MailField.toField(sortField.getListField()));
+                fp = Conversations.getFetchProfileConversationByHeaders(examineHasAttachmentUserFlags, previewMode, null == sortField ? MailField.RECEIVED_DATE : MailField.toField(sortField.getListField()));
             }
-            List<Conversation> conversations = ThreadSorts.getConversationList(imapMessageStorage.getImapFolder(), getSortRange(lookAhead, messageCount, order), isRev1, fp, imapMessageStorage.getImapServerInfo(), examineHasAttachmentUserFlags, previewSupported, searchTerm);
+            List<Conversation> conversations = ThreadSorts.getConversationList(imapMessageStorage.getImapFolder(), getSortRange(lookAhead, messageCount, order), isRev1, fp, imapMessageStorage.getImapServerInfo(), examineHasAttachmentUserFlags, userAndContext, previewMode, searchTerm);
             // Merge with sent folder
             {
                 // Switch folder
                 imapMessageStorage.openReadOnly(sentFullName);
-                List<MailMessage> sentMessages = Conversations.messagesFor(imapMessageStorage.getImapFolder(), lookAhead, order, fp, imapMessageStorage.getImapServerInfo(), byEnvelope, examineHasAttachmentUserFlags, previewSupported);
+                List<MailMessage> sentMessages = Conversations.messagesFor(imapMessageStorage.getImapFolder(), lookAhead, order, fp, imapMessageStorage.getImapServerInfo(), byEnvelope, examineHasAttachmentUserFlags, userAndContext, previewMode);
                 if (false == sentMessages.isEmpty()) {
                     // Filter messages already contained in conversations
                     {
@@ -801,7 +808,7 @@ public final class IMAPConversationWorker {
             }
             // Fill selected chunk
             if (!list.isEmpty()) {
-                FetchProfile fetchProfile = IMAPMessageStorage.checkFetchProfile(getFetchProfile(usedFields.toArray(), headerNames, null, null, true, examineHasAttachmentUserFlags, previewSupported));
+                FetchProfile fetchProfile = IMAPMessageStorage.checkFetchProfile(getFetchProfile(usedFields.toArray(), headerNames, null, null, true, examineHasAttachmentUserFlags && usedFields.containsAny(MailField.ATTACHMENT, MailField.CONTENT_TYPE), previewMode));
                 List<MailMessage> msgs = new LinkedList<>();
                 List<MailMessage> sentmsgs = new LinkedList<>();
                 for (List<MailMessage> conversation : list) {
@@ -813,15 +820,15 @@ public final class IMAPConversationWorker {
                         }
                     }
                 }
-                new MailMessageFillerIMAPCommand(msgs, isRev1, fetchProfile, imapMessageStorage.getImapServerInfo(), examineHasAttachmentUserFlags, previewSupported, imapMessageStorage.getImapFolder()).doCommand();
+                new MailMessageFillerIMAPCommand(msgs, isRev1, fetchProfile, imapMessageStorage.getImapServerInfo(), examineHasAttachmentUserFlags, userAndContext, previewMode, imapMessageStorage.getImapFolder()).doCommand();
                 if (!sentmsgs.isEmpty()) {
                     // Switch folder
                     imapMessageStorage.openReadOnly(sentFullName);
-                    new MailMessageFillerIMAPCommand(sentmsgs, isRev1, fetchProfile, imapMessageStorage.getImapServerInfo(), examineHasAttachmentUserFlags, previewSupported, imapMessageStorage.getImapFolder()).doCommand();
+                    new MailMessageFillerIMAPCommand(sentmsgs, isRev1, fetchProfile, imapMessageStorage.getImapServerInfo(), examineHasAttachmentUserFlags, userAndContext, previewMode, imapMessageStorage.getImapFolder()).doCommand();
                 }
             }
         } else {
-            list = ThreadSorts.getConversations(imapMessageStorage.getImapFolder(), getSortRange(lookAhead, messageCount, order), isRev1, examineHasAttachmentUserFlags, previewSupported, imapMessageStorage.getImapServerInfo(), searchTerm, null == sortField ? MailField.RECEIVED_DATE : MailField.toField(sortField.getListField()));
+            list = ThreadSorts.getConversations(imapMessageStorage.getImapFolder(), getSortRange(lookAhead, messageCount, order), isRev1, examineHasAttachmentUserFlags, userAndContext, previewMode, imapMessageStorage.getImapServerInfo(), searchTerm, null == sortField ? MailField.RECEIVED_DATE : MailField.toField(sortField.getListField()));
             // Sort root elements
             {
                 final MailSortField effectiveSortField = null == sortField ? MailSortField.RECEIVED_DATE : sortField;
@@ -859,7 +866,7 @@ public final class IMAPConversationWorker {
                 for (List<MailMessage> conversation : list) {
                     msgs.addAll(conversation);
                 }
-                new MailMessageFillerIMAPCommand(msgs, isRev1, getFetchProfile(usedFields.toArray(), headerNames, null, null, true, examineHasAttachmentUserFlags, previewSupported), imapMessageStorage.getImapServerInfo(), examineHasAttachmentUserFlags, previewSupported, imapMessageStorage.getImapFolder()).doCommand();
+                new MailMessageFillerIMAPCommand(msgs, isRev1, getFetchProfile(usedFields.toArray(), headerNames, null, null, true, examineHasAttachmentUserFlags && usedFields.containsAny(MailField.ATTACHMENT, MailField.CONTENT_TYPE), previewMode), imapMessageStorage.getImapServerInfo(), examineHasAttachmentUserFlags, userAndContext, previewMode, imapMessageStorage.getImapFolder()).doCommand();
             }
         }
         // Apply account identifier
@@ -1101,11 +1108,12 @@ public final class IMAPConversationWorker {
         // Fill requested fields/headers
         {
             boolean examineHasAttachmentUserFlags = imapMessageStorage.examineHasAttachmentUserFlags;
-            boolean previewSupported = imapMessageStorage.previewSupported;
-            FetchProfile fetchProfile = IMAPMessageStorage.checkFetchProfile(getFetchProfile(usedFields.toArray(), headerNames, null, null, true, examineHasAttachmentUserFlags, previewSupported));
+            IUserAndContext userAndContext = imapMessageStorage.session;
+            PreviewMode previewMode = imapMessageStorage.previewMode;
+            FetchProfile fetchProfile = IMAPMessageStorage.checkFetchProfile(getFetchProfile(usedFields.toArray(), headerNames, null, null, true, examineHasAttachmentUserFlags && usedFields.containsAny(MailField.ATTACHMENT, MailField.CONTENT_TYPE), previewMode));
             boolean isRev1 = imapMessageStorage.getImapConfig().getImapCapabilities().hasIMAP4rev1();
             TLongObjectMap<MailMessage> messages = parseResult.getMessages();
-            new MailMessageFillerIMAPCommand(messages.valueCollection(), isRev1, fetchProfile, imapMessageStorage.getImapServerInfo(), examineHasAttachmentUserFlags, previewSupported, imapMessageStorage.getImapFolder()).doCommand();
+            new MailMessageFillerIMAPCommand(messages.valueCollection(), isRev1, fetchProfile, imapMessageStorage.getImapServerInfo(), examineHasAttachmentUserFlags, userAndContext, previewMode, imapMessageStorage.getImapFolder()).doCommand();
             imapMessageStorage.setAccountInfo(messages.values(new MailMessage[messages.size()]));
         }
 
@@ -1229,13 +1237,14 @@ public final class IMAPConversationWorker {
             // Add sort field
             usedFields.add(MailField.toField(effectiveSortField.getListField()));
             boolean examineHasAttachmentUserFlags = imapMessageStorage.examineHasAttachmentUserFlags;
-            boolean previewSupported = imapMessageStorage.previewSupported;
-            final FetchProfile fetchProfile = IMAPMessageStorage.checkFetchProfile(getFetchProfile(usedFields.toArray(), imapMessageStorage.getIMAPProperties().isFastFetch(), examineHasAttachmentUserFlags, previewSupported));
+            IUserAndContext userAndContext = imapMessageStorage.session;
+            PreviewMode previewMode = imapMessageStorage.previewMode;
+            final FetchProfile fetchProfile = IMAPMessageStorage.checkFetchProfile(getFetchProfile(usedFields.toArray(), imapMessageStorage.getIMAPProperties().isFastFetch(), examineHasAttachmentUserFlags && usedFields.containsAny(MailField.ATTACHMENT, MailField.CONTENT_TYPE), previewMode));
             final boolean body = usedFields.contains(MailField.BODY) || usedFields.contains(MailField.FULL);
             if (!body) {
                 final Map<MessageInfo, MailMessage> mapping;
                 {
-                    TLongObjectMap<MailMessage> messages = new SimpleFetchIMAPCommand(imapMessageStorage.getImapFolder(), imapMessageStorage.getImapConfig().getImapCapabilities().hasIMAP4rev1(), MessageInfo.toSeqNums(messageIds).toArray(), fetchProfile, imapMessageStorage.getImapServerInfo(), examineHasAttachmentUserFlags, previewSupported).doCommand();
+                    TLongObjectMap<MailMessage> messages = new SimpleFetchIMAPCommand(imapMessageStorage.getImapFolder(), imapMessageStorage.getImapConfig().getImapCapabilities().hasIMAP4rev1(), MessageInfo.toSeqNums(messageIds).toArray(), fetchProfile, imapMessageStorage.getImapServerInfo(), examineHasAttachmentUserFlags, userAndContext, previewMode).doCommand();
                     mapping = new HashMap<>(messages.size());
                     messages.forEachEntry(new TLongObjectProcedure<MailMessage>() {
 
@@ -1285,7 +1294,7 @@ public final class IMAPConversationWorker {
             /*
              * Include body
              */
-            Message[] msgs = new MessageFetchIMAPCommand(imapMessageStorage.getImapFolder(), imapMessageStorage.getImapConfig().getImapCapabilities().hasIMAP4rev1(), MessageInfo.toSeqNums(messageIds), fetchProfile, imapMessageStorage.getImapServerInfo(), false, true, body).doCommand();
+            Message[] msgs = new MessageFetchIMAPCommand(imapMessageStorage.getImapFolder(), imapMessageStorage.getImapConfig().getImapCapabilities().hasIMAP4rev1(), MessageInfo.toSeqNums(messageIds), fetchProfile, imapMessageStorage.getImapServerInfo(), false, true, body, previewMode).doCommand();
             /*
              * Apply thread level
              */
