@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -112,7 +112,6 @@ public class IMAPDefaultFolderChecker {
     protected final IMAPAccess imapAccess;
     protected final IMAPConfig imapConfig;
     protected final boolean hasMetadata;
-    protected final boolean ignoreSubscription;
 
     /**
      * Initializes a new {@link IMAPDefaultFolderChecker}.
@@ -132,7 +131,6 @@ public class IMAPDefaultFolderChecker {
         this.imapAccess = imapAccess;
         imapConfig = imapAccess.getIMAPConfig();
         this.hasMetadata = hasMetadata;
-        ignoreSubscription = imapConfig.getIMAPProperties().isIgnoreSubscription();
     }
 
     /**
@@ -320,7 +318,7 @@ public class IMAPDefaultFolderChecker {
                              * changes to the database now influence the default folders.
                              */
                             boolean reinitSpecialUseIfLoaded = false;
-                            ListLsubEntry entry = ListLsubCache.getCachedLISTEntry(INBOX, accountId, tmp, session, ignoreSubscription, reinitSpecialUseIfLoaded);
+                            ListLsubEntry entry = ListLsubCache.getCachedLISTEntry(INBOX, accountId, tmp, session, reinitSpecialUseIfLoaded, imapConfig.getIMAPProperties());
 
                             if (entry.exists()) {
                                 inboxFolder = tmp;
@@ -332,11 +330,16 @@ public class IMAPDefaultFolderChecker {
                                 try {
                                     IMAPCommandsCollection.createFolder(tmp, sep, FOLDER_TYPE);
                                 } catch (@SuppressWarnings("unused") MessagingException e) {
-                                    IMAPCommandsCollection.createFolder(tmp, sep, Folder.HOLDS_MESSAGES);
+                                    try {
+                                        IMAPCommandsCollection.createFolder(tmp, sep, Folder.HOLDS_MESSAGES);
+                                    } catch (MessagingException x) {
+                                        // Folder creation failed
+                                        throw new FolderCreationFailedException(tmp.getFullName(), "IMAP folder could not be created", x);
+                                    }
                                 }
-                                ListLsubCache.addSingle(INBOX, accountId, tmp, session, ignoreSubscription);
+                                ListLsubCache.addSingle(INBOX, accountId, tmp, session, imapConfig.getIMAPProperties());
                                 inboxFolder = (IMAPFolder) imapStore.getFolder(INBOX);
-                                entry = ListLsubCache.getCachedLISTEntry(INBOX, accountId, inboxFolder, session, ignoreSubscription);
+                                entry = ListLsubCache.getCachedLISTEntry(INBOX, accountId, inboxFolder, session, imapConfig.getIMAPProperties().isIgnoreSubscription(), imapConfig.getIMAPProperties());
                             }
                             inboxListEntry = entry;
                         }
@@ -345,8 +348,8 @@ public class IMAPDefaultFolderChecker {
                              * Subscribe INBOX folder
                              */
                             inboxFolder.setSubscribed(true);
-                            ListLsubCache.addSingle(INBOX, accountId, inboxFolder, session, ignoreSubscription);
-                            inboxListEntry = ListLsubCache.getCachedLISTEntry(INBOX, accountId, inboxFolder, session, ignoreSubscription);
+                            ListLsubCache.addSingle(INBOX, accountId, inboxFolder, session, imapConfig.getIMAPProperties());
+                            inboxListEntry = ListLsubCache.getCachedLISTEntry(INBOX, accountId, inboxFolder, session, imapConfig.getIMAPProperties().isIgnoreSubscription(), imapConfig.getIMAPProperties());
                         }
                         char sep = inboxListEntry.getSeparator();
                         /*
@@ -457,7 +460,7 @@ public class IMAPDefaultFolderChecker {
                     return null;
                 }
                 // Check entry
-                ListLsubEntry cached = ListLsubCache.getCachedLISTEntry(entry.getFullName(), accountId, imapStore, session, ignoreSubscription);
+                ListLsubEntry cached = ListLsubCache.getCachedLISTEntry(entry.getFullName(), accountId, imapStore, session, imapConfig.getIMAPProperties());
                 if (!cached.exists()) {
                     LOG.warn("{} SPECIAL-USE marked folder \"{}\" does not exist. Skipping... (user={}, context={})", SPECIAL_USES[index], entry.getFullName(), Integer.valueOf(session.getUserId()), Integer.valueOf(session.getContextId()));
                     return Boolean.FALSE;
@@ -491,7 +494,7 @@ public class IMAPDefaultFolderChecker {
             boolean isPrimary = MailAccount.DEFAULT_ID == accountId;
 
             // Entries with "\Drafts" marker
-            Collection<ListLsubEntry> entries = ListLsubCache.getDraftsEntry(accountId, imapFolder, session, ignoreSubscription);
+            Collection<ListLsubEntry> entries = ListLsubCache.getDraftsEntry(accountId, imapFolder, session, imapConfig.getIMAPProperties());
             int size = entries.size();
             if (size > 0) {
                 int index = StorageUtility.INDEX_DRAFTS;
@@ -502,7 +505,7 @@ public class IMAPDefaultFolderChecker {
             }
 
             // Entries with "\Junk" marker
-            entries = ListLsubCache.getJunkEntry(accountId, imapFolder, session, ignoreSubscription);
+            entries = ListLsubCache.getJunkEntry(accountId, imapFolder, session, imapConfig.getIMAPProperties());
             size = entries.size();
             if (size > 0) {
                 int index = StorageUtility.INDEX_SPAM;
@@ -513,7 +516,7 @@ public class IMAPDefaultFolderChecker {
             }
 
             // Entries with "\Send" marker
-            entries = ListLsubCache.getSentEntry(accountId, imapFolder, session, ignoreSubscription);
+            entries = ListLsubCache.getSentEntry(accountId, imapFolder, session, imapConfig.getIMAPProperties());
             size = entries.size();
             if (size > 0) {
                 int index = StorageUtility.INDEX_SENT;
@@ -524,7 +527,7 @@ public class IMAPDefaultFolderChecker {
             }
 
             // Entries with "\Trash" marker
-            entries = ListLsubCache.getTrashEntry(accountId, imapFolder, session, ignoreSubscription);
+            entries = ListLsubCache.getTrashEntry(accountId, imapFolder, session, imapConfig.getIMAPProperties());
             size = entries.size();
             if (size > 0) {
                 int index = StorageUtility.INDEX_TRASH;
@@ -580,9 +583,9 @@ public class IMAPDefaultFolderChecker {
                 ConfigView view = viewFactory.getView(session.getUserId(), session.getContextId());
                 ConfigProperty<Boolean> prop = view.property(ConfigViewScope.USER.getScopeName(), "com.openexchange.mail.specialuse.check", Boolean.class);
 
-                if (prop.isDefined()) {
-                    Boolean b = prop.get();
-                    checkSpecialUseFolder = null != b && b.booleanValue();
+                Boolean b = prop.get();
+                if (b != null) {
+                    checkSpecialUseFolder = b.booleanValue();
                     prop.set(null);
                 }
             }
@@ -843,7 +846,7 @@ public class IMAPDefaultFolderChecker {
         int prefixLen = prefix.length();
         String desiredFullName = prefixLen == 0 ? qualifiedName : new StringBuilder(prefix).append(qualifiedName).toString();
         {
-            ListLsubEntry entry = modified.getValue() ? ListLsubCache.getActualLISTEntry(desiredFullName, accountId, imapStore, session, ignoreSubscription) : ListLsubCache.getCachedLISTEntry(desiredFullName, accountId, imapStore, session, ignoreSubscription);
+            ListLsubEntry entry = modified.getValue() ? ListLsubCache.getActualLISTEntry(desiredFullName, accountId, imapStore, session, imapConfig.getIMAPProperties()) : ListLsubCache.getCachedLISTEntry(desiredFullName, accountId, imapStore, session, imapConfig.getIMAPProperties());
             if (null != entry && entry.exists()) {
                 // The easy one -- already existing; just check subscription status
                 boolean checkSpecialUseForExisting = false;
