@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -139,4 +139,14 @@ public final class IMAPExceptionMessages implements LocalizableStrings {
 
     public static final String MAX_NUMBER_OF_MESSAGES_EXCEEDED_MSG = "Too many messages requested (limit is %1$s). Please query a smaller range.";
 
+    public static final String TOO_MANY_FOLDERS_MSG = "Mail folder \"%1$s\" could not be created since there are too many folders. Please delete a folder in order to create a new one.";
+
+    public static final String SUBSCRIBE_AFTER_CREATE_FAILED_MSG = "Mail folder \"%1$s\" could be successfully created, but subscription of that folder failed.";
+
+    // User attempts to change an IMAP folder permission that is associated with a deputy
+    public static final String NO_DEPUTY_PERMISSION_CHANGE = "A deputy permission must not be changed";
+
+    // User attempts to remove an IMAP folder permission that is associated with a deputy
+    public static final String NO_DEPUTY_PERMISSION_DELETE = "A deputy permission must not be removed";
+
 }
