@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -42,11 +42,13 @@ import com.openexchange.imap.cache.ListLsubEntry;
 import com.openexchange.imap.cache.ListLsubRuntimeException;
 import com.openexchange.imap.cache.RightsCache;
 import com.openexchange.imap.config.IMAPConfig;
+import com.openexchange.imap.config.IMAPProperties;
 import com.openexchange.imap.config.IMAPReloadable;
 import com.openexchange.imap.services.Services;
 import com.openexchange.java.Strings;
 import com.openexchange.log.LogProperties;
 import com.openexchange.mail.MailExceptionCode;
+import com.openexchange.mail.PreviewMode;
 import com.openexchange.mail.api.MailMessageStorage;
 import com.openexchange.mail.api.enhanced.MailMessageStorageLong;
 import com.openexchange.mail.dataobjects.MailFolder;
@@ -174,9 +176,8 @@ public abstract class IMAPFolderWorker extends MailMessageStorageLong {
     protected IMAPFolder imapFolder;
     protected final Set<IMAPFolder> otherFolders;
     protected int holdsMessages = -1;
-    protected final boolean ignoreSubscriptions;
     protected final boolean examineHasAttachmentUserFlags;
-    protected final boolean previewSupported;
+    protected final PreviewMode previewMode;
 
     /**
      * Initializes a new {@link IMAPFolderWorker}.
@@ -198,9 +199,8 @@ public abstract class IMAPFolderWorker extends MailMessageStorageLong {
         imapServerInfo = IMAPServerInfo.instanceFor(imapConfig, accountId);
         aclExtension = imapConfig.getACLExtension();
         otherFolders = new HashSet<IMAPFolder>(4);
-        ignoreSubscriptions = imapConfig.getIMAPProperties().isIgnoreSubscription();
         examineHasAttachmentUserFlags = imapConfig.getCapabilities().hasAttachmentMarker();
-        previewSupported = imapConfig.asMap().containsKey(IMAPCapabilities.CAP_TEXT_PREVIEW_NEW);
+        previewMode = IMAPProperties.previewModeFor(imapConfig);
     }
 
     /**
@@ -383,7 +383,7 @@ public abstract class IMAPFolderWorker extends MailMessageStorageLong {
      */
     protected boolean holdsMessages() throws OXException, MessagingException {
         if (holdsMessages == -1) {
-            holdsMessages = ListLsubCache.getCachedLISTEntry(imapFolder.getFullName(), accountId, imapFolder, session, this.ignoreSubscriptions).canOpen() ? 1 : 0;
+            holdsMessages = ListLsubCache.getCachedLISTEntry(imapFolder.getFullName(), accountId, imapFolder, session, imapConfig.getIMAPProperties()).canOpen() ? 1 : 0;
         }
         return holdsMessages > 0;
     }
@@ -506,14 +506,14 @@ public abstract class IMAPFolderWorker extends MailMessageStorageLong {
             boolean openIt = true;
 
             // Get associated LIST entry and verify existence
-            ListLsubEntry listEntry = ListLsubCache.getCachedLISTEntry(fullName, accountId, retval, session, ignoreSubscriptions);
+            ListLsubEntry listEntry = ListLsubCache.getCachedLISTEntry(fullName, accountId, retval, session, imapConfig.getIMAPProperties());
             if (!isRootFolder && !"INBOX".equals(fullName) && (!listEntry.exists())) {
                 // Try to open the folder although not LISTed to check existence
                 if (!tryOpen(retval, desiredMode)) {
                     throw IMAPException.create(IMAPException.Code.FOLDER_NOT_FOUND, imapConfig, session, fullName);
                 }
                 openIt = false;
-                listEntry = ListLsubCache.addSingleByFolder(accountId, retval, session, ignoreSubscriptions);
+                listEntry = ListLsubCache.addSingleByFolder(accountId, retval, session, imapConfig.getIMAPProperties());
             }
 
             // Check if selectable aka. does hold messages
