@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -26,6 +26,7 @@ import java.net.UnknownHostException;
 import java.util.concurrent.ConcurrentMap;
 import org.cliffc.high_scale_lib.NonBlockingHashMap;
 import com.openexchange.exception.OXException;
+import com.openexchange.java.InetAddresses;
 import com.openexchange.mail.Protocol;
 
 /**
@@ -105,7 +106,7 @@ public final class IMAPProtocol extends Protocol {
             return false;
         }
         try {
-            return (null == concurrentMap.putIfAbsent(InetAddress.getByName(host), Integer.valueOf(maxCount)));
+            return (null == concurrentMap.putIfAbsent(InetAddresses.forString(host), Integer.valueOf(maxCount)));
         } catch (UnknownHostException e) {
             LOG.warn("Couldn't resolve host name: {}. Assume default max-count setting instead.", host, e);
             return false;
@@ -123,7 +124,7 @@ public final class IMAPProtocol extends Protocol {
             return;
         }
         try {
-            concurrentMap.remove(InetAddress.getByName(host));
+            concurrentMap.remove(InetAddresses.forString(host));
         } catch (UnknownHostException e) {
             LOG.warn("Couldn't remove max-count setting for: {}", host, e);
         }
@@ -144,7 +145,7 @@ public final class IMAPProtocol extends Protocol {
             return null == thisMaxCount ? -1 : thisMaxCount.intValue();
         }
         try {
-            final Integer mc = concurrentMap.get(InetAddress.getByName(host));
+            final Integer mc = concurrentMap.get(InetAddresses.forString(host));
             return mc == null ? (null == thisMaxCount ? -1 : thisMaxCount.intValue()) : minOf(mc.intValue(), thisMaxCount);
         } catch (UnknownHostException e) {
             LOG.warn("Couldn't resolve host name: {}. Return default max-count setting instead.", host, e);
