@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -37,7 +37,14 @@ import com.openexchange.mail.search.SearchTerm;
  */
 public class ImapConfigSearchTermVisitor extends AbstractSearchTermVisitor{
 
-    boolean needsSourroundingAndTerm = true;
+    private boolean needsSourroundingAndTerm = true;
+    
+    /**
+     * Initializes a new {@link ImapConfigSearchTermVisitor}.
+     */
+    public ImapConfigSearchTermVisitor() {
+        super();
+    }
 
     @Override
     public void visit(ANDTerm term) {
