@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -57,6 +57,10 @@ public enum MailCacheCode {
 	 * LIST/LSUB collection
 	 */
 	LIST_LSUB(7),
+    /**
+     * Namespaces
+     */
+    NAMESPACES(8),
 	;
 
 	private final int code;
