@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -25,6 +25,7 @@ import static com.openexchange.imap.IMAPCommandsCollection.canCreateSubfolder;
 import java.util.concurrent.Callable;
 import java.util.concurrent.ExecutionException;
 import java.util.concurrent.TimeUnit;
+import java.util.concurrent.atomic.AtomicReference;
 import javax.mail.MessagingException;
 import javax.mail.Store;
 import com.google.common.cache.Cache;
@@ -32,9 +33,10 @@ import com.google.common.cache.CacheBuilder;
 import com.openexchange.exception.OXException;
 import com.openexchange.imap.config.IMAPConfig;
 import com.openexchange.imap.config.IMAPProperties;
+import com.openexchange.imap.namespace.Namespace;
 import com.openexchange.mail.MailExceptionCode;
 import com.openexchange.mail.mime.MimeMailException;
-import com.openexchange.mailaccount.MailAccount;
+import com.openexchange.mailaccount.Account;
 import com.openexchange.session.Session;
 import com.sun.mail.imap.DefaultFolder;
 import com.sun.mail.imap.IMAPStore;
@@ -48,7 +50,7 @@ public final class RootSubfoldersEnabledCache {
 
     private static final String ROOT_FULL_NAME = "";
 
-    private static volatile Cache<String, Boolean> CACHE;
+    private static final AtomicReference<Cache<String, Boolean>> CACHE_REFERENCE = new AtomicReference<>();
 
     /**
      * Initializes a new {@link RootSubfoldersEnabledCache}.
@@ -61,35 +63,32 @@ public final class RootSubfoldersEnabledCache {
      * Initializes this cache.
      */
     public static void init() {
-        if (CACHE == null) {
-            synchronized (RootSubfoldersEnabledCache.class) {
-                if (CACHE == null) {
-                    CACHE = CacheBuilder.newBuilder().expireAfterAccess(2, TimeUnit.DAYS).build();
-                }
-            }
-        }
+        initElseGet();
+    }
+
+    private static Cache<String, Boolean> initElseGet() {
+        Cache<String, Boolean> cache = CacheBuilder.newBuilder().expireAfterAccess(2, TimeUnit.DAYS).build();
+        return CACHE_REFERENCE.compareAndSet(null, cache) ? cache : CACHE_REFERENCE.get();
     }
 
     /**
      * Tear-down for this cache.
      */
     public static void tearDown() {
-        if (CACHE != null) {
-            synchronized (RootSubfoldersEnabledCache.class) {
-                Cache<String, Boolean> cache = CACHE;
-                if (cache != null) {
-                    clear(cache);
-                    CACHE = null;
-                }
-            }
+        Cache<String, Boolean> cache = CACHE_REFERENCE.getAndSet(null);
+        if (cache != null) {
+            cache.invalidateAll();
         }
     }
 
     /**
-     * Clears this cache.
+     * Clears given cache.
      */
-    private static void clear(Cache<String, Boolean> map) {
-        map.invalidateAll();
+    private static void clear() {
+        Cache<String, Boolean> cache = CACHE_REFERENCE.getAndSet(null);
+        if (cache != null) {
+            cache.invalidateAll();
+        }
     }
 
     private static String getKeyFor(Store store, IMAPConfig imapConfig, boolean namespacePerUser) {
@@ -109,7 +108,7 @@ public final class RootSubfoldersEnabledCache {
      * @throws OXException If a mail error occurs
      */
     public static boolean isRootSubfoldersEnabled(IMAPConfig imapConfig, IMAPStore imapStore, Session session) throws OXException {
-        if (MailAccount.DEFAULT_ID == imapConfig.getAccountId()) {
+        if (Account.DEFAULT_ID == imapConfig.getAccountId()) {
             Boolean rootSubfoldersAllowed = IMAPProperties.getInstance().areRootSubfoldersAllowed(session.getUserId(), session.getContextId());
             if (null != rootSubfoldersAllowed) {
                 return rootSubfoldersAllowed.booleanValue();
@@ -118,8 +117,8 @@ public final class RootSubfoldersEnabledCache {
 
         try {
             // Check for personal namespace
-            String personalNamespace = NamespaceFoldersCache.getPersonalNamespace(imapStore, true, session, imapConfig.getAccountId());
-            if (ROOT_FULL_NAME.equals(personalNamespace)) {
+            Namespace personalNamespace = NamespacesCache.getPersonalNamespace(imapStore, true, session, imapConfig.getAccountId());
+            if (ROOT_FULL_NAME.equals(personalNamespace == null ? null : personalNamespace.getFullName())) {
                 // Root level is signaled as personal namespace, thus creating folder there SHOULD be possible...
                 return true;
             }
@@ -141,7 +140,7 @@ public final class RootSubfoldersEnabledCache {
      * @throws OXException If a mail error occurs
      */
     public static boolean isRootSubfoldersEnabled(IMAPConfig imapConfig, DefaultFolder imapDefaultFolder, Session session) throws OXException {
-        if (MailAccount.DEFAULT_ID == imapConfig.getAccountId()) {
+        if (Account.DEFAULT_ID == imapConfig.getAccountId()) {
             Boolean rootSubfoldersAllowed = IMAPProperties.getInstance().areRootSubfoldersAllowed(session.getUserId(), session.getContextId());
             if (null != rootSubfoldersAllowed) {
                 return rootSubfoldersAllowed.booleanValue();
@@ -151,8 +150,8 @@ public final class RootSubfoldersEnabledCache {
         IMAPStore store = (IMAPStore) imapDefaultFolder.getStore();
         try {
             // Check for personal namespace
-            String personalNamespace = NamespaceFoldersCache.getPersonalNamespace(store, true, session, imapConfig.getAccountId());
-            if (ROOT_FULL_NAME.equals(personalNamespace)) {
+            Namespace personalNamespace = NamespacesCache.getPersonalNamespace(store, true, session, imapConfig.getAccountId());
+            if (ROOT_FULL_NAME.equals(personalNamespace == null ? null : personalNamespace.getFullName())) {
                 // Root level is signaled as personal namespace, thus creating folder there SHOULD be possible...
                 return true;
             }
@@ -168,7 +167,7 @@ public final class RootSubfoldersEnabledCache {
      * Checks if root sub-folders capability is enabled for given IMAP account.
      */
     private static boolean isRootSubfoldersEnabled0(String key, IMAPConfig imapConfig, DefaultFolder imapDefaultFolder, boolean namespacePerUser) throws OXException {
-        Cache<String, Boolean> cache = CACHE;
+        Cache<String, Boolean> cache = CACHE_REFERENCE.get();
         Boolean rootSubfoldersEnabled = cache.getIfPresent(key);
         if (null != rootSubfoldersEnabled) {
             return rootSubfoldersEnabled.booleanValue();
