@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -83,7 +83,7 @@ public class ListLsubInterner {
     // -------------------------------------------------------------------------------------------------------------------------------------
 
     private final Interner<String> fullNameInterner;
-    private final Interner<String> attribteInterner;
+    private final Interner<String> attributeInterner;
 
     /**
      * Initializes a new {@link ListLsubInterner}.
@@ -94,7 +94,7 @@ public class ListLsubInterner {
         super();
         if (useInterner) {
             fullNameInterner = javax.mail.util.Interners.getFullNameInterner();
-            attribteInterner = javax.mail.util.Interners.getAttributeInterner();
+            attributeInterner = javax.mail.util.Interners.getAttributeInterner();
         } else {
             Interner<String> noopInterner = new Interner<String>() {
 
@@ -104,7 +104,7 @@ public class ListLsubInterner {
                 }
             };
             fullNameInterner = noopInterner;
-            attribteInterner = noopInterner;
+            attributeInterner = noopInterner;
         }
     }
 
@@ -122,8 +122,8 @@ public class ListLsubInterner {
      *
      * @return The interner
      */
-    public Interner<String> getAttribteInterner() {
-        return attribteInterner;
+    public Interner<String> getAttributeInterner() {
+        return attributeInterner;
     }
 
 }
