@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -75,7 +75,7 @@ public abstract class AbstractIMAPCommand<T> {
     }
 
     private static final class IMAPCommandResponseInterceptor implements ResponseInterceptor {
-        
+
         private final AbstractIMAPCommand<?> abstractIMAPCommand;
         private MessagingException exception;
 
@@ -86,9 +86,9 @@ public abstract class AbstractIMAPCommand<T> {
 
         @Override
         public boolean intercept(Response response) {
-            if (exception == null) {                
+            if (exception == null) {
                 try {
-                    return abstractIMAPCommand.addLoopCondition() ? abstractIMAPCommand.handleResponse(response) : false;
+                    return abstractIMAPCommand.addLoopCondition() && abstractIMAPCommand.handleResponse(response);
                 } catch (MessagingException e) {
                     exception = e;
                 }
@@ -126,10 +126,17 @@ public abstract class AbstractIMAPCommand<T> {
             Response[] r = null;
             Response response = null;
             for (int argsIndex = 0; argsIndex < args.length; argsIndex++) {
-                String imapCmd = abstractIMAPCommand.getCommand(argsIndex);
+                CommandAndArgument imapCmd = abstractIMAPCommand.getCommand(argsIndex, protocol);
                 // Use interceptor for FETCH commands
-                IMAPCommandResponseInterceptor nullableInterceptor = imapCmd.indexOf("FETCH ") >= 0 ? new IMAPCommandResponseInterceptor(abstractIMAPCommand) : null;
-                r = performCommand(protocol, imapCmd, null, Optional.ofNullable(nullableInterceptor), false);
+                IMAPCommandResponseInterceptor nullableInterceptor = null;
+                if (abstractIMAPCommand.isFetchCommand() || imapCmd.getCommand().indexOf("FETCH ") >= 0) {
+                    nullableInterceptor = new IMAPCommandResponseInterceptor(abstractIMAPCommand);
+                    LogProperties.putProperty(LogProperties.Name.MAIL_COMMAND, prepareImapCommandForLogging(imapCmd.getCommand()));
+                    if (null != imapFolder) {
+                        LogProperties.putProperty(LogProperties.Name.MAIL_FULL_NAME, imapFolder.getFullName());
+                    }
+                }
+                r = performCommand(protocol, imapCmd.getCommand(), imapCmd.getArgs(), Optional.ofNullable(nullableInterceptor), false);
                 response = r[r.length - 1];
                 if (response.isOK()) {
                     try {
@@ -158,7 +165,7 @@ public abstract class AbstractIMAPCommand<T> {
                     if (ImapUtility.isInvalidMessageset(response)) {
                         return abstractIMAPCommand.getDefaultValue();
                     }
-                    LogProperties.putProperty(LogProperties.Name.MAIL_COMMAND, prepareImapCommandForLogging(imapCmd));
+                    LogProperties.putProperty(LogProperties.Name.MAIL_COMMAND, prepareImapCommandForLogging(imapCmd.getCommand()));
                     if (null != imapFolder) {
                         LogProperties.putProperty(LogProperties.Name.MAIL_FULL_NAME, imapFolder.getFullName());
                     }
@@ -178,14 +185,14 @@ public abstract class AbstractIMAPCommand<T> {
                          */
                         return abstractIMAPCommand.getDefaultValue();
                     }
-                    LogProperties.putProperty(LogProperties.Name.MAIL_COMMAND, prepareImapCommandForLogging(imapCmd));
+                    LogProperties.putProperty(LogProperties.Name.MAIL_COMMAND, prepareImapCommandForLogging(imapCmd.getCommand()));
                     if (null != imapFolder) {
                         LogProperties.putProperty(LogProperties.Name.MAIL_FULL_NAME, imapFolder.getFullName());
                     }
                     throw new CommandFailedException(response);
                 } else {
                     notifyResponseHandlersSafe(r, protocol);
-                    LogProperties.putProperty(LogProperties.Name.MAIL_COMMAND, prepareImapCommandForLogging(imapCmd));
+                    LogProperties.putProperty(LogProperties.Name.MAIL_COMMAND, prepareImapCommandForLogging(imapCmd.getCommand()));
                     protocol.handleResult(response);
                 }
             }
@@ -232,13 +239,23 @@ public abstract class AbstractIMAPCommand<T> {
         return null;
     }
 
+    /**
+     * Checks if this IMAP command issues an IMAP <code>FETCH</code> command.
+     *
+     * @return <code>true</code> if an IMAP <code>FETCH</code> command is issued; otherwise <code>false</code>
+     */
+    protected boolean isFetchCommand() {
+        return false;
+    }
+
     /**
      * Gets the IMAP command to be executed.
      *
-     * @param argsIndex - the argument index
-     * @return the IMAP command to be executed
+     * @param argsIndex The argument index
+     * @param protocol The IMAP protocol with which the command will be executed
+     * @return The IMAP command to be executed
      */
-    protected abstract String getCommand(int argsIndex);
+    protected abstract CommandAndArgument getCommand(int argsIndex, IMAPProtocol protocol);
 
     /**
      * Gets the IMAP command's arguments whereas each argument <b>must not</b> exceed 16384 bytes.
