@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -21,12 +21,15 @@
 
 package com.openexchange.imap.command;
 
-import static com.openexchange.imap.IMAPCommandsCollection.prepareStringArgument;
+import java.nio.charset.StandardCharsets;
 import java.util.Arrays;
 import javax.mail.MessagingException;
 import com.openexchange.java.Strings;
+import com.sun.mail.iap.Argument;
 import com.sun.mail.iap.Response;
 import com.sun.mail.imap.IMAPFolder;
+import com.sun.mail.imap.protocol.BASE64MailboxEncoder;
+import com.sun.mail.imap.protocol.IMAPProtocol;
 
 /**
  * {@link CopyIMAPCommand} - Copies messages from given folder to given destination folder just using their sequence numbers/UIDs
@@ -111,7 +114,7 @@ public final class CopyIMAPCommand extends AbstractIMAPCommand<long[]> {
             returnDefaultValue = (uids.length == 0);
         }
         this.fast = fast;
-        this.destFolderName = prepareStringArgument(destFolderName);
+        this.destFolderName = destFolderName;
         length = uids.length;
         args = length == 0 ? ARGS_EMPTY : (isSequential ? new String[] { new StringBuilder(64).append(uids[0]).append(':').append(
             uids[length - 1]).toString() } : IMAPNumArgSplitter.splitUIDArg(
@@ -144,7 +147,7 @@ public final class CopyIMAPCommand extends AbstractIMAPCommand<long[]> {
         fast = true;
         uid = false;
         uids = DEFAULT_RETVAL;
-        this.destFolderName = prepareStringArgument(destFolderName);
+        this.destFolderName = destFolderName;
         retval = DEFAULT_RETVAL;
         args = 1 == messageCount ? ARGS_FIRST : ARGS_ALL;
         length = -1;
@@ -161,15 +164,22 @@ public final class CopyIMAPCommand extends AbstractIMAPCommand<long[]> {
     }
 
     @Override
-    protected String getCommand(int argsIndex) {
-        final StringBuilder sb = new StringBuilder(args[argsIndex].length() + 64);
+    protected CommandAndArgument getCommand(int argsIndex, IMAPProtocol protocol) {
+        final StringBuilder sb = new StringBuilder(8);
         if (uid) {
             sb.append("UID ");
         }
-        sb.append("COPY ");
-        sb.append(args[argsIndex]);
-        sb.append(' ').append(destFolderName);
-        return sb.toString();
+        sb.append("COPY");
+
+        Argument arguments = new Argument();
+        arguments.writeAtom(args[argsIndex]);
+        if (protocol.supportsUtf8()) {
+            arguments.writeString(destFolderName, StandardCharsets.UTF_8);
+        } else {
+            // encode the mbox as per RFC2060
+            arguments.writeString(BASE64MailboxEncoder.encode(destFolderName));
+        }
+        return new CommandAndArgument(sb.toString(), arguments);
     }
 
     @Override
