@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -26,6 +26,7 @@ import javax.mail.Flags.Flag;
 import javax.mail.MessagingException;
 import com.sun.mail.iap.Response;
 import com.sun.mail.imap.IMAPFolder;
+import com.sun.mail.imap.protocol.IMAPProtocol;
 
 /**
  * {@link FlagsIMAPCommand} - Enables/disables message's system e.g. \SEEN or \DELETED and user flags as well.
@@ -232,7 +233,7 @@ public final class FlagsIMAPCommand extends AbstractIMAPCommand<Boolean> {
     }
 
     @Override
-    protected String getCommand(int argsIndex) {
+    protected CommandAndArgument getCommand(int argsIndex, IMAPProtocol protocol) {
         // UID STORE %s %sFLAGS (%s)
         final StringBuilder sb = new StringBuilder(args[argsIndex].length() + 64);
         if (uid) {
@@ -246,7 +247,7 @@ public final class FlagsIMAPCommand extends AbstractIMAPCommand<Boolean> {
             sb.append(".SILENT");
         }
         sb.append(" (").append(flagsStr).append(')');
-        return sb.toString();
+        return CommandAndArgument.instanceFor(sb.toString());
     }
 
     @Override
