@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -25,6 +25,7 @@ import static com.openexchange.java.Autoboxing.I;
 import java.util.ArrayList;
 import java.util.List;
 import javax.mail.Message;
+import com.openexchange.java.Strings;
 import gnu.trove.list.TIntList;
 import gnu.trove.list.array.TIntArrayList;
 import gnu.trove.list.array.TLongArrayList;
@@ -170,7 +171,7 @@ public final class IMAPNumArgSplitter {
             }
         }
         if (l.isEmpty()) {
-            return new String[0];
+            return Strings.getEmptyStrings();
         }
         if (!keepOrder) {
             l.sort();
@@ -201,7 +202,7 @@ public final class IMAPNumArgSplitter {
             }
         }
         if (l.isEmpty()) {
-            return new String[0];
+            return Strings.getEmptyStrings();
         }
         if (!keepOrder) {
             l.sort();
@@ -229,7 +230,7 @@ public final class IMAPNumArgSplitter {
             }
         }
         if (l.isEmpty()) {
-            return new String[0];
+            return Strings.getEmptyStrings();
         }
         if (!keepOrder) {
             l.sort();
@@ -349,7 +350,7 @@ public final class IMAPNumArgSplitter {
         final int len = numArg.length();
         if (len <= maxLen) {
             if (len == 0) {
-                return new String[0];
+                return Strings.getEmptyStrings();
             }
             return new String[] { numArg };
         }
