@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -31,12 +31,15 @@ import org.slf4j.Logger;
 import com.openexchange.exception.OXException;
 import com.openexchange.imap.IMAPCommandsCollection;
 import com.openexchange.imap.IMAPServerInfo;
+import com.openexchange.mail.PreviewMode;
 import com.openexchange.mail.dataobjects.IDMailMessage;
 import com.openexchange.mail.dataobjects.MailMessage;
 import com.openexchange.mail.utils.StorageUtility;
+import com.openexchange.session.IUserAndContext;
 import com.sun.mail.iap.Response;
 import com.sun.mail.imap.IMAPFolder;
 import com.sun.mail.imap.protocol.FetchResponse;
+import com.sun.mail.imap.protocol.IMAPProtocol;
 import com.sun.mail.imap.protocol.UID;
 import gnu.trove.list.TLongList;
 import gnu.trove.list.array.TLongArrayList;
@@ -64,6 +67,7 @@ public final class MailMessageFillerIMAPCommand extends AbstractIMAPCommand<Void
     private final String fullname;
     private final int index;
     private final boolean examineHasAttachmentUserFlags;
+    private final IUserAndContext userAndContext;
 
     /**
      * Initializes a new {@link MailMessageFillerIMAPCommand}.
@@ -73,13 +77,14 @@ public final class MailMessageFillerIMAPCommand extends AbstractIMAPCommand<Void
      * @param fp The fetch profile to use
      * @param serverInfo The IMAP server information deduced from configuration
      * @param examineHasAttachmentUserFlags Whether has-attachment user flags should be considered
-     * @param previewSupported Whether target IMAP server supports <code>"PREVIEW=FUZZY"</code> capability
+     * @param previewMode Whether target IMAP server supports any preview> capability
      * @param imapFolder The IMAP folder providing connected protocol
      * @throws MessagingException If initialization fails
      */
-    public MailMessageFillerIMAPCommand(Collection<MailMessage> messages, boolean isRev1, FetchProfile fp, IMAPServerInfo serverInfo, boolean examineHasAttachmentUserFlags, boolean previewSupported, IMAPFolder imapFolder) throws MessagingException {
+    public MailMessageFillerIMAPCommand(Collection<MailMessage> messages, boolean isRev1, FetchProfile fp, IMAPServerInfo serverInfo, boolean examineHasAttachmentUserFlags, IUserAndContext userAndContext, PreviewMode previewMode, IMAPFolder imapFolder) throws MessagingException {
         super(imapFolder);
         this.examineHasAttachmentUserFlags = examineHasAttachmentUserFlags;
+        this.userAndContext = userAndContext;
         final int messageCount = imapFolder.getMessageCount();
         if (messageCount <= 0) {
             returnDefaultValue = true;
@@ -100,11 +105,11 @@ public final class MailMessageFillerIMAPCommand extends AbstractIMAPCommand<Void
         this.messages = tm;
         this.uids = tuids.toArray();
         if (length == messageCount) {
-            command = getFetchCommand(isRev1, checkFetchProfile(fp), false, serverInfo, previewSupported);
+            command = getFetchCommand(isRev1, checkFetchProfile(fp), false, serverInfo, previewMode);
             args = (1 == length ? ARGS_FIRST : ARGS_ALL);
             uid = false;
         } else {
-            command = getFetchCommand(isRev1, checkFetchProfile(fp), false, serverInfo, previewSupported);
+            command = getFetchCommand(isRev1, checkFetchProfile(fp), false, serverInfo, previewMode);
             args = IMAPNumArgSplitter.splitUIDArg(uids, false, LENGTH_WITH_UID + command.length());
             uid = true;
         }
@@ -125,7 +130,7 @@ public final class MailMessageFillerIMAPCommand extends AbstractIMAPCommand<Void
     }
 
     @Override
-    protected String getCommand(int argsIndex) {
+    protected CommandAndArgument getCommand(int argsIndex, IMAPProtocol protocol) {
         final StringBuilder sb = new StringBuilder(args[argsIndex].length() + 64);
         if (uid) {
             sb.append("UID ");
@@ -133,7 +138,12 @@ public final class MailMessageFillerIMAPCommand extends AbstractIMAPCommand<Void
         sb.append("FETCH ");
         sb.append(args[argsIndex]);
         sb.append(" (").append(command).append(')');
-        return sb.toString();
+        return CommandAndArgument.instanceFor(sb.toString());
+    }
+
+    @Override
+    protected boolean isFetchCommand() {
+        return true;
     }
 
     @Override
@@ -151,7 +161,7 @@ public final class MailMessageFillerIMAPCommand extends AbstractIMAPCommand<Void
         /*
          * Response is null or not a FetchResponse
          */
-        if (!FetchResponse.class.isInstance(currentReponse)) {
+        if (!(currentReponse instanceof FetchResponse)) {
             return false;
         }
         final FetchResponse fetchResponse = (FetchResponse) currentReponse;
@@ -162,7 +172,7 @@ public final class MailMessageFillerIMAPCommand extends AbstractIMAPCommand<Void
                 try {
                     IDMailMessage idm = (IDMailMessage) message;
                     idm.setAccountId(accountId);
-                    MailMessageFetchIMAPCommand.handleFetchRespone(idm, fetchResponse, fullname, examineHasAttachmentUserFlags);
+                    MailMessageFetchIMAPCommand.handleFetchRespone(idm, fetchResponse, fullname, examineHasAttachmentUserFlags, userAndContext);
                 } catch (OXException e) {
                     /*
                      * Discard corrupt message
