@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -26,6 +26,7 @@ import javax.mail.MessagingException;
 import com.sun.mail.iap.Response;
 import com.sun.mail.imap.IMAPFolder;
 import com.sun.mail.imap.protocol.FetchResponse;
+import com.sun.mail.imap.protocol.IMAPProtocol;
 import com.sun.mail.imap.protocol.UID;
 import gnu.trove.list.TLongList;
 import gnu.trove.list.array.TLongArrayList;
@@ -77,12 +78,17 @@ public final class MessageUIDsIMAPCommand extends AbstractIMAPCommand<long[]> {
     }
 
     @Override
-    protected String getCommand(int argsIndex) {
+    protected CommandAndArgument getCommand(int argsIndex, IMAPProtocol protocol) {
         final StringBuilder sb = new StringBuilder(args[argsIndex].length() + 64);
         sb.append("FETCH ");
         sb.append(args[argsIndex]);
         sb.append(" (UID)");
-        return sb.toString();
+        return CommandAndArgument.instanceFor(sb.toString());
+    }
+
+    @Override
+    protected boolean isFetchCommand() {
+        return true;
     }
 
     private static final long[] EMPTY_ARR = new long[0];
