@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -25,6 +25,7 @@ import javax.mail.MessagingException;
 import com.sun.mail.iap.Response;
 import com.sun.mail.imap.IMAPFolder;
 import com.sun.mail.imap.protocol.FetchResponse;
+import com.sun.mail.imap.protocol.IMAPProtocol;
 import com.sun.mail.imap.protocol.UID;
 import gnu.trove.list.TIntList;
 import gnu.trove.list.array.TIntArrayList;
@@ -58,8 +59,15 @@ public final class SeqNumIMAPCommand extends AbstractIMAPCommand<int[]> {
         this.uids = uids == null ? L1 : uids;
         returnDefaultValue = (this.uids.length == 0);
         length = this.uids.length;
-        args = length == 0 ? ARGS_EMPTY : (isSequential ? new String[] { new StringBuilder(64).append(this.uids[0]).append(':').append(
-            this.uids[this.uids.length - 1]).toString() } : IMAPNumArgSplitter.splitUIDArg(this.uids, true, -1));
+        if (length == 0) {
+            args = ARGS_EMPTY;
+        } else {
+            if (isSequential) {
+                args = new String[] { new StringBuilder(64).append(this.uids[0]).append(':').append(this.uids[this.uids.length - 1]).toString() };
+            } else {
+                args = IMAPNumArgSplitter.splitUIDArg(this.uids, true, -1);
+            }
+        }
         sia = new TIntArrayList(length);
     }
 
@@ -74,12 +82,17 @@ public final class SeqNumIMAPCommand extends AbstractIMAPCommand<int[]> {
     }
 
     @Override
-    protected String getCommand(int argsIndex) {
+    protected CommandAndArgument getCommand(int argsIndex, IMAPProtocol protocol) {
         final StringBuilder sb = new StringBuilder(args[argsIndex].length() + 64);
         sb.append("UID FETCH ");
         sb.append(args[argsIndex]);
         sb.append(" (UID)");
-        return sb.toString();
+        return CommandAndArgument.instanceFor(sb.toString());
+    }
+
+    @Override
+    protected boolean isFetchCommand() {
+        return true;
     }
 
     private static final int[] EMPTY_ARR = new int[0];
