@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -25,7 +25,6 @@ import static com.openexchange.java.Autoboxing.I;
 import java.io.IOException;
 import java.io.InputStream;
 import java.text.ParseException;
-import java.util.Collection;
 import java.util.Enumeration;
 import java.util.HashSet;
 import java.util.Map;
@@ -40,19 +39,24 @@ import javax.mail.Part;
 import javax.mail.UIDFolder;
 import javax.mail.internet.InternetAddress;
 import javax.mail.internet.InternetHeaders;
-import javax.mail.internet.MailDateFormat;
 import com.google.common.collect.ImmutableMap;
 import com.openexchange.exception.OXException;
 import com.openexchange.imap.IMAPException;
 import com.openexchange.imap.IMAPServerInfo;
 import com.openexchange.mail.FullnameArgument;
+import com.openexchange.mail.PreviewMode;
 import com.openexchange.mail.dataobjects.IDMailMessage;
 import com.openexchange.mail.dataobjects.MailMessage;
 import com.openexchange.mail.mime.ContentType;
+import com.openexchange.mail.mime.LockableMailDateFormat;
 import com.openexchange.mail.mime.MessageHeaders;
 import com.openexchange.mail.mime.MimeMailException;
 import com.openexchange.mail.mime.MimeTypes;
+import com.openexchange.mail.mime.converters.MimeMessageConverter;
 import com.openexchange.mail.mime.utils.MimeMessageUtility;
+import com.openexchange.mail.utils.DateUtils;
+import com.openexchange.session.IUserAndContext;
+import com.openexchange.session.UserAndContext;
 import com.sun.mail.iap.Response;
 import com.sun.mail.imap.IMAPFolder;
 import com.sun.mail.imap.protocol.BODY;
@@ -60,6 +64,7 @@ import com.sun.mail.imap.protocol.BODYSTRUCTURE;
 import com.sun.mail.imap.protocol.ENVELOPE;
 import com.sun.mail.imap.protocol.FLAGS;
 import com.sun.mail.imap.protocol.FetchResponse;
+import com.sun.mail.imap.protocol.IMAPProtocol;
 import com.sun.mail.imap.protocol.INTERNALDATE;
 import com.sun.mail.imap.protocol.Item;
 import com.sun.mail.imap.protocol.RFC822DATA;
@@ -96,6 +101,7 @@ public final class SimpleFetchIMAPCommand extends AbstractIMAPCommand<TLongObjec
     private int index;
     private boolean determineAttachmentByHeader;
     private final boolean examineHasAttachmentUserFlags;
+    private final IUserAndContext optUserAndContext;
     private final String fullname;
     private final Set<FetchItemHandler> lastHandlers;
     private final TLongObjectMap<MailMessage> map;
@@ -109,18 +115,19 @@ public final class SimpleFetchIMAPCommand extends AbstractIMAPCommand<TLongObjec
      * @param fp The fetch profile to use
      * @param serverInfo The IMAP server information
      * @param examineHasAttachmentUserFlags Whether has-attachment user flags should be considered
-     * @param previewSupported Whether target IMAP server supports <code>"PREVIEW=FUZZY"</code> capability
+     * @param previewMode Whether target IMAP server supports any preview capability
      * @throws MessagingException If initialization fails
      */
-    public SimpleFetchIMAPCommand(IMAPFolder imapFolder, boolean isRev1, int[] seqNums, FetchProfile fp, IMAPServerInfo serverInfo, boolean examineHasAttachmentUserFlags, boolean previewSupported) throws MessagingException {
+    public SimpleFetchIMAPCommand(IMAPFolder imapFolder, boolean isRev1, int[] seqNums, FetchProfile fp, IMAPServerInfo serverInfo, boolean examineHasAttachmentUserFlags, IUserAndContext optUserAndContext, PreviewMode previewMode) throws MessagingException {
         super(imapFolder);
+        this.optUserAndContext = optUserAndContext;
         final int messageCount = imapFolder.getMessageCount();
         if (messageCount <= 0) {
             returnDefaultValue = true;
         }
         this.examineHasAttachmentUserFlags = examineHasAttachmentUserFlags;
         lastHandlers = new HashSet<FetchItemHandler>();
-        command = getFetchCommand(isRev1, fp, false, serverInfo, previewSupported);
+        command = getFetchCommand(isRev1, fp, false, serverInfo, previewMode);
         uid = false;
         length = seqNums.length;
         map = new TLongObjectHashMap<MailMessage>(length);
@@ -140,26 +147,27 @@ public final class SimpleFetchIMAPCommand extends AbstractIMAPCommand<TLongObjec
      * @param fetchProfile The fetch profile to use
      * @param serverInfo The IMAP server information
      * @param examineHasAttachmentUserFlags Whether has-attachment user flags should be considered
-     * @param previewSupported Whether target IMAP server supports <code>"PREVIEW=FUZZY"</code> capability
+     * @param previewMode Whether target IMAP server supports any preview capability
      * @throws MessagingException If initialization fails
      */
-    public SimpleFetchIMAPCommand(IMAPFolder imapFolder, boolean isRev1, long[] uids, FetchProfile fetchProfile, IMAPServerInfo imapServerInfo, boolean examineHasAttachmentUserFlags, boolean previewSupported) throws MessagingException {
+    public SimpleFetchIMAPCommand(IMAPFolder imapFolder, boolean isRev1, long[] uids, FetchProfile fetchProfile, IMAPServerInfo imapServerInfo, boolean examineHasAttachmentUserFlags, UserAndContext optUserAndContext, PreviewMode previewMode) throws MessagingException {
         super(imapFolder);
         final int messageCount = imapFolder.getMessageCount();
         if (messageCount <= 0) {
             returnDefaultValue = true;
         }
         this.examineHasAttachmentUserFlags = examineHasAttachmentUserFlags;
+        this.optUserAndContext = optUserAndContext;
         lastHandlers = new HashSet<FetchItemHandler>();
         length = uids.length;
         map = new TLongObjectHashMap<MailMessage>(length);
         if (length == messageCount) {
             fetchProfile.add(UIDFolder.FetchProfileItem.UID);
-            command = getFetchCommand(isRev1, fetchProfile, false, imapServerInfo, previewSupported);
+            command = getFetchCommand(isRev1, fetchProfile, false, imapServerInfo, previewMode);
             args = (1 == length ? ARGS_FIRST : ARGS_ALL);
             uid = false;
         } else {
-            command = getFetchCommand(isRev1, fetchProfile, false, imapServerInfo, previewSupported);
+            command = getFetchCommand(isRev1, fetchProfile, false, imapServerInfo, previewMode);
             args = IMAPNumArgSplitter.splitUIDArg(uids, false, LENGTH_WITH_UID + command.length());
             uid = true;
         }
@@ -217,7 +225,7 @@ public final class SimpleFetchIMAPCommand extends AbstractIMAPCommand<TLongObjec
     }
 
     @Override
-    protected String getCommand(int argsIndex) {
+    protected CommandAndArgument getCommand(int argsIndex, IMAPProtocol protocol) {
         final StringBuilder sb = new StringBuilder(args[argsIndex].length() + 64);
         if (uid) {
             sb.append("UID ");
@@ -225,7 +233,12 @@ public final class SimpleFetchIMAPCommand extends AbstractIMAPCommand<TLongObjec
         sb.append("FETCH ");
         sb.append(args[argsIndex]);
         sb.append(" (").append(command).append(')');
-        return sb.toString();
+        return CommandAndArgument.instanceFor(sb.toString());
+    }
+
+    @Override
+    protected boolean isFetchCommand() {
+        return true;
     }
 
     @Override
@@ -275,7 +288,7 @@ public final class SimpleFetchIMAPCommand extends AbstractIMAPCommand<TLongObjec
                 } else {
                     FetchItemHandler itemHandler = MAP.get(item.getClass());
                     if (null == itemHandler) {
-                        itemHandler = getItemHandlerByItem(item, examineHasAttachmentUserFlags);
+                        itemHandler = getItemHandlerByItem(item, examineHasAttachmentUserFlags, optUserAndContext);
                         if (null == itemHandler) {
                             LOG.warn("Unknown FETCH item: {}", item.getClass().getName());
                         } else {
@@ -292,7 +305,7 @@ public final class SimpleFetchIMAPCommand extends AbstractIMAPCommand<TLongObjec
             if (null != delayed) {
                 FetchItemHandler itemHandler = MAP.get(delayed.getClass());
                 if (null == itemHandler) {
-                    itemHandler = getItemHandlerByItem(delayed, examineHasAttachmentUserFlags);
+                    itemHandler = getItemHandlerByItem(delayed, examineHasAttachmentUserFlags, optUserAndContext);
                     if (null == itemHandler) {
                         LOG.warn("Unknown FETCH item: {}", delayed.getClass().getName());
                     } else {
@@ -333,7 +346,7 @@ public final class SimpleFetchIMAPCommand extends AbstractIMAPCommand<TLongObjec
         return true;
     }
 
-    private static FetchItemHandler getItemHandlerByItem(Item item, boolean examineHasAttachmentUserFlags) {
+    private static FetchItemHandler getItemHandlerByItem(Item item, boolean examineHasAttachmentUserFlags, IUserAndContext optUserAndContext) {
         if ((item instanceof RFC822DATA) || (item instanceof BODY)) {
             return HEADER_ITEM_HANDLER;
         } else if (item instanceof UID) {
@@ -341,7 +354,7 @@ public final class SimpleFetchIMAPCommand extends AbstractIMAPCommand<TLongObjec
         } else if (item instanceof INTERNALDATE) {
             return INTERNALDATE_ITEM_HANDLER;
         } else if (item instanceof Flags) {
-            return new FLAGSFetchItemHandler(examineHasAttachmentUserFlags);
+            return new FLAGSFetchItemHandler(examineHasAttachmentUserFlags, optUserAndContext);
         } else if (item instanceof ENVELOPE) {
             return ENVELOPE_ITEM_HANDLER;
         } else if (item instanceof RFC822SIZE) {
@@ -416,6 +429,13 @@ public final class SimpleFetchIMAPCommand extends AbstractIMAPCommand<TLongObjec
                     mailMessage.addBcc(MimeMessageUtility.getAddressHeader(hdr.getValue()));
                 }
             })
+            .put(MessageHeaders.HDR_SENDER, new HeaderHandler() {
+
+                @Override
+                public void handle(Header hdr, IDMailMessage mailMessage) throws OXException {
+                    mailMessage.addSender(MimeMessageUtility.getAddressHeader(hdr.getValue()));
+                }
+            })
             .put(MessageHeaders.HDR_REPLY_TO, new HeaderHandler() {
 
                 @Override
@@ -441,8 +461,17 @@ public final class SimpleFetchIMAPCommand extends AbstractIMAPCommand<TLongObjec
 
                 @Override
                 public void handle(Header hdr, IDMailMessage mailMessage) throws OXException {
-                    final MailDateFormat mdf = MimeMessageUtility.getDefaultMailDateFormat();
-                    synchronized (mdf) {
+                    LockableMailDateFormat mdf = MimeMessageUtility.getDefaultMailDateFormat();
+                    if (mdf.getLock().tryLock()) {
+                        try {
+                            mailMessage.setSentDate(mdf.parse(hdr.getValue()));
+                        } catch (ParseException e) {
+                            LOG.error("", e);
+                        } finally {
+                            mdf.getLock().unlock();
+                        }
+                    } else {
+                        mdf = DateUtils.initMailDateFormat();
                         try {
                             mailMessage.setSentDate(mdf.parse(hdr.getValue()));
                         } catch (ParseException e) {
@@ -553,144 +582,34 @@ public final class SimpleFetchIMAPCommand extends AbstractIMAPCommand<TLongObjec
     private static final class FLAGSFetchItemHandler implements FetchItemHandler {
 
         private final boolean examineHasAttachmentUserFlags;
+        private final IUserAndContext optUserAndContext;
 
-        FLAGSFetchItemHandler(boolean examineHasAttachmentUserFlags) {
+        FLAGSFetchItemHandler(boolean examineHasAttachmentUserFlags, IUserAndContext optUserAndContext) {
             super();
             this.examineHasAttachmentUserFlags = examineHasAttachmentUserFlags;
+            this.optUserAndContext = optUserAndContext;
         }
 
         @Override
         public void handleItem(Item item, IDMailMessage msg, org.slf4j.Logger logger) throws MessagingException {
             final FLAGS flags = (FLAGS) item;
-            /*
-             * Parse system flags
-             */
-            int retval = 0;
-            int colorLabel = MailMessage.COLOR_LABEL_NONE;
-            Collection<String> ufCol = null;
-            if (flags.contains(Flags.Flag.ANSWERED)) {
-                retval |= MailMessage.FLAG_ANSWERED;
-            }
-            if (flags.contains(Flags.Flag.DELETED)) {
-                retval |= MailMessage.FLAG_DELETED;
-            }
-            if (flags.contains(Flags.Flag.DRAFT)) {
-                retval |= MailMessage.FLAG_DRAFT;
-            }
-            if (flags.contains(Flags.Flag.FLAGGED)) {
-                retval |= MailMessage.FLAG_FLAGGED;
-            }
-            if (flags.contains(Flags.Flag.RECENT)) {
-                retval |= MailMessage.FLAG_RECENT;
-            }
-            if (flags.contains(Flags.Flag.SEEN)) {
-                retval |= MailMessage.FLAG_SEEN;
-            }
-            if (flags.contains(Flags.Flag.USER)) {
-                retval |= MailMessage.FLAG_USER;
-            }
-            final String[] userFlags = flags.getUserFlags();
-            if (userFlags != null) {
-                /*
-                 * Mark message to contain user flags
-                 */
-                final Set<String> set = new HashSet<String>(userFlags.length);
-                for (String userFlag : userFlags) {
-                    if (MailMessage.isColorLabel(userFlag)) {
-                        try {
-                            colorLabel = MailMessage.getColorLabelIntValue(userFlag);
-                        } catch (@SuppressWarnings("unused") final OXException e) {
-                            // Cannot occur
-                            colorLabel = MailMessage.COLOR_LABEL_NONE;
-                        }
-                    } else if (MailMessage.USER_FORWARDED.equalsIgnoreCase(userFlag)) {
-                        retval |= MailMessage.FLAG_FORWARDED;
-                    } else if (MailMessage.USER_READ_ACK.equalsIgnoreCase(userFlag)) {
-                        retval |= MailMessage.FLAG_READ_ACK;
-                    } else if (examineHasAttachmentUserFlags && MailMessage.USER_HAS_ATTACHMENT.equalsIgnoreCase(userFlag)) {
-                        msg.setHasAttachment(true);
-                    } else if (examineHasAttachmentUserFlags && MailMessage.USER_HAS_NO_ATTACHMENT.equalsIgnoreCase(userFlag)) {
-                        msg.setHasAttachment(false);
-                    } else {
-                        set.add(userFlag);
-                    }
-                }
-                ufCol = set.isEmpty() ? null : set;
-            }
-            /*
-             * Apply parsed flags
-             */
-            msg.setFlags(retval);
-            msg.setColorLabel(colorLabel);
-            if (null != ufCol) {
-                msg.addUserFlags(ufCol.toArray(new String[ufCol.size()]));
+            if (optUserAndContext == null) {
+                MimeMessageConverter.parseFlags(flags, examineHasAttachmentUserFlags, msg);
+            } else {
+                MimeMessageConverter.parseFlags(flags, examineHasAttachmentUserFlags, optUserAndContext.getUserId(), optUserAndContext.getContextId(), msg);
             }
         }
 
         @Override
         public void handleMessage(Message message, IDMailMessage msg, org.slf4j.Logger logger) throws MessagingException {
             final Flags flags = message.getFlags();
-            /*
-             * Parse system flags
-             */
-            int retval = 0;
-            int colorLabel = MailMessage.COLOR_LABEL_NONE;
-            Collection<String> ufCol = null;
-            if (flags.contains(Flags.Flag.ANSWERED)) {
-                retval |= MailMessage.FLAG_ANSWERED;
-            }
-            if (flags.contains(Flags.Flag.DELETED)) {
-                retval |= MailMessage.FLAG_DELETED;
-            }
-            if (flags.contains(Flags.Flag.DRAFT)) {
-                retval |= MailMessage.FLAG_DRAFT;
-            }
-            if (flags.contains(Flags.Flag.FLAGGED)) {
-                retval |= MailMessage.FLAG_FLAGGED;
-            }
-            if (flags.contains(Flags.Flag.RECENT)) {
-                retval |= MailMessage.FLAG_RECENT;
-            }
-            if (flags.contains(Flags.Flag.SEEN)) {
-                retval |= MailMessage.FLAG_SEEN;
-            }
-            if (flags.contains(Flags.Flag.USER)) {
-                retval |= MailMessage.FLAG_USER;
-            }
-            final String[] userFlags = flags.getUserFlags();
-            if (userFlags != null) {
-                /*
-                 * Mark message to contain user flags
-                 */
-                final Set<String> set = new HashSet<String>(userFlags.length);
-                for (String userFlag : userFlags) {
-                    if (MailMessage.isColorLabel(userFlag)) {
-                        try {
-                            colorLabel = MailMessage.getColorLabelIntValue(userFlag);
-                        } catch (@SuppressWarnings("unused") final OXException e) {
-                            // Cannot occur
-                            colorLabel = MailMessage.COLOR_LABEL_NONE;
-                        }
-                    } else if (MailMessage.USER_FORWARDED.equalsIgnoreCase(userFlag)) {
-                        retval |= MailMessage.FLAG_FORWARDED;
-                    } else if (MailMessage.USER_READ_ACK.equalsIgnoreCase(userFlag)) {
-                        retval |= MailMessage.FLAG_READ_ACK;
-                    } else {
-                        set.add(userFlag);
-                    }
-                }
-                ufCol = set.isEmpty() ? null : set;
-            }
-            /*
-             * Apply parsed flags
-             */
-            msg.setFlags(retval);
-            msg.setColorLabel(colorLabel);
-            if (null != ufCol) {
-                msg.addUserFlags(ufCol.toArray(new String[ufCol.size()]));
+            if (optUserAndContext == null) {
+                MimeMessageConverter.parseFlags(flags, examineHasAttachmentUserFlags, msg);
+            } else {
+                MimeMessageConverter.parseFlags(flags, examineHasAttachmentUserFlags, optUserAndContext.getUserId(), optUserAndContext.getContextId(), msg);
             }
         }
-    };
+    }
 
     private static final FetchItemHandler ENVELOPE_ITEM_HANDLER = new FetchItemHandler() {
 
@@ -702,6 +621,7 @@ public final class SimpleFetchIMAPCommand extends AbstractIMAPCommand<TLongObjec
             msg.addCc(env.cc);
             msg.addBcc(env.bcc);
             msg.addReplyTo(env.replyTo);
+            msg.addSender(env.sender);
             msg.addHeader("In-Reply-To", env.inReplyTo);
             msg.addHeader("Message-Id", env.messageId);
             msg.setSubject(MimeMessageUtility.decodeEnvelopeSubject(env.subject), true);
@@ -727,8 +647,24 @@ public final class SimpleFetchIMAPCommand extends AbstractIMAPCommand<TLongObjec
             if (null != header && header.length > 0) {
                 msg.setSubject(MimeMessageUtility.decodeMultiEncodedHeader(header[0]), true);
             }
+            header = message.getHeader("Sender");
+            if (null != header && header.length > 0) {
+                msg.addSender(MimeMessageUtility.getAddressHeader(header[0]));
+            }
             msg.setSentDate(message.getSentDate());
         }
+
+        private String addrs2String(InternetAddress[] addrs) {
+            if (null == addrs || addrs.length == 0) {
+                return null;
+            }
+            StringBuilder sb = new StringBuilder(addrs.length * 16);
+            sb.append(addrs[0].toString());
+            for (int i = 1; i < addrs.length; i++) {
+                sb.append(", ").append(addrs[i].toString());
+            }
+            return sb.toString();
+        }
     };
 
     private static final FetchItemHandler INTERNALDATE_ITEM_HANDLER = new FetchItemHandler() {
@@ -893,11 +829,11 @@ public final class SimpleFetchIMAPCommand extends AbstractIMAPCommand<TLongObjec
      * @param fp The fetch profile to convert
      * @param loadBody <code>true</code> if message body should be loaded; otherwise <code>false</code>
      * @param serverInfo The IMAP server information
-     * @param previewSupported Whether target IMAP server supports <code>"PREVIEW=FUZZY"</code> capability
+     * @param previewMode Whether target IMAP server supports any preview capability
      * @return The FETCH items to craft a FETCH command
      */
-    private static String getFetchCommand(boolean isRev1, FetchProfile fp, boolean loadBody, IMAPServerInfo serverInfo, boolean previewSupported) {
-        return MailMessageFetchIMAPCommand.getFetchCommand(isRev1, fp, loadBody, serverInfo, previewSupported);
+    private static String getFetchCommand(boolean isRev1, FetchProfile fp, boolean loadBody, IMAPServerInfo serverInfo, PreviewMode previewMode) {
+        return MailMessageFetchIMAPCommand.getFetchCommand(isRev1, fp, loadBody, serverInfo, previewMode);
     }
 
     /**
