@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -24,6 +24,7 @@ package com.openexchange.imap.command;
 import javax.mail.MessagingException;
 import com.sun.mail.iap.Response;
 import com.sun.mail.imap.IMAPFolder;
+import com.sun.mail.imap.protocol.IMAPProtocol;
 
 /**
  * {@link SimpleIMAPCommand}
@@ -75,13 +76,13 @@ public final class SimpleIMAPCommand extends AbstractIMAPCommand<Boolean> {
     }
 
     @Override
-    protected String getCommand(int argsIndex) {
+    protected CommandAndArgument getCommand(int argsIndex, IMAPProtocol protocol) {
         final StringBuilder sb = new StringBuilder(args[argsIndex].length() + 64);
         sb.append(command);
         if (!java.util.Arrays.equals(ARGS_EMPTY, args)) {
             sb.append(args[argsIndex]);
         }
-        return sb.toString();
+        return CommandAndArgument.instanceFor(sb.toString());
     }
 
     @Override
