@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -53,7 +53,7 @@ import io.micrometer.core.instrument.Timer;
  */
 public class MonitoringCommandExecutor implements CommandExecutor {
 
-    private static final List<Class<? extends Exception>> NETWORK_COMMUNICATION_ERRORS = ImmutableList.of(
+    private static final List<Class<? extends Throwable>> NETWORK_COMMUNICATION_ERRORS = ImmutableList.of(
         com.sun.mail.iap.ConnectionException.class,
         com.sun.mail.iap.ByeIOException.class,
         java.net.SocketTimeoutException.class,
