@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -23,6 +23,7 @@ package com.openexchange.imap.config;
 
 import java.util.Map;
 import java.util.Set;
+import com.openexchange.mail.PreviewMode;
 import com.openexchange.mail.api.IMailProperties;
 import com.openexchange.mail.api.MailConfig.BoolCapVal;
 
@@ -61,6 +62,13 @@ public interface IIMAPProperties extends IMailProperties {
      */
     public boolean isDebugLogEnabled();
 
+    /**
+     * Gets the server pattern to check for when debug (traffic) log is enabled to trace IMAP communication.
+     *
+     * @return The server pattern
+     */
+    public String getDebugServerPattern();
+
     /**
      * Whether the pre-login capabilities are supposed to be overwritten (completely replaced with the ones advertised after login)
      *
@@ -166,6 +174,13 @@ public interface IIMAPProperties extends IMailProperties {
      */
     public boolean allowFolderCaches();
 
+    /**
+     * Gets the time-to-live in milliseconds for an initialized folder cache.
+     *
+     * @return The time-to-live in milliseconds for an initialized folder cache
+     */
+    public long getFolderCacheTimeoutMillis();
+
     /**
      * Checks whether it is allowed to FETCH single headers
      *
@@ -195,6 +210,13 @@ public interface IIMAPProperties extends IMailProperties {
      */
     public boolean isAttachmentMarkerEnabled();
 
+    /**
+     * Gets the optional preferred preview mode.
+     *
+     * @return The preview mode or <code>null</code>
+     */
+    public PreviewMode getPreferredPreviewMode();
+
     /**
      * Gets the map holding IMAP servers with new ACL Extension.
      *
