@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -39,11 +39,14 @@ import com.openexchange.imap.acl.ACLExtension;
 import com.openexchange.imap.acl.ACLExtensionFactory;
 import com.openexchange.imap.cache.CapabilitiesCache;
 import com.openexchange.imap.cache.CapabilitiesCache.CapabilitiesResponse;
+import com.openexchange.java.InetAddresses;
 import com.openexchange.mail.api.IMailProperties;
 import com.openexchange.mail.api.MailCapabilities;
 import com.openexchange.mail.api.MailConfig;
 import com.openexchange.mail.api.UrlInfo;
 import com.openexchange.mail.config.MailConfigException;
+import com.openexchange.mailaccount.Account;
+import com.openexchange.mailaccount.MailAccounts;
 import com.openexchange.session.Session;
 import com.openexchange.tools.net.URIDefaults;
 import com.openexchange.tools.net.URIParser;
@@ -213,7 +216,6 @@ public final class IMAPConfig extends MailConfig {
      * @param session The session possibly caching capabilities information
      * @throws OXException If IMAP capabilities cannot be initialized
      */
-    @SuppressWarnings("unchecked")
     public void initializeCapabilities(IMAPStore imapStore, Session session) throws OXException {
         if (imapCapabilities == null) {
             synchronized (this) {
@@ -225,19 +227,21 @@ public final class IMAPConfig extends MailConfig {
                     final CapabilitiesResponse response = CapabilitiesCache.getCapabilitiesResponse(imapStore, this, session, accountId);
 
                     IMAPCapabilities imapCaps = response.getImapCapabilities();
-                    Map<String, String> caps = imapStore.getCapabilities();
+                    Map<String, String> caps = response.getMap();
 
                     BoolCapVal supportsACLs = IMAPProperties.getInstance().getSupportsACLs();
-                    if (BoolCapVal.FALSE.equals(supportsACLs)) {
+                    if (BoolCapVal.FALSE == supportsACLs) {
                         imapCaps.setACL(false);
                         caps = new HashMap<String, String>(caps);
                         caps.remove("ACL");
                         caps = Collections.unmodifiableMap(caps);
-                    } else if (BoolCapVal.TRUE.equals(supportsACLs)) {
-                        imapCaps.setACL(true);
-                        caps = new HashMap<String, String>(caps);
-                        caps.put("ACL", "ACL");
-                        caps = Collections.unmodifiableMap(caps);
+                    } else if (BoolCapVal.TRUE == supportsACLs) {
+                        if (Account.DEFAULT_ID == accountId || MailAccounts.isSecondaryAccount(accountId, session)) { // NOSONARLINT
+                            imapCaps.setACL(true);
+                            caps = new HashMap<String, String>(caps);
+                            caps.put("ACL", "ACL");
+                            caps = Collections.unmodifiableMap(caps);
+                        }
                     }
 
                     imapCapabilities = imapCaps;
@@ -313,11 +317,13 @@ public final class IMAPConfig extends MailConfig {
             return acl.booleanValue();
         }
         final BoolCapVal supportsACLs = IMAPProperties.getInstance().getSupportsACLs();
-        final Map<String, String> capabilities = this.capabilities;
-        if (capabilities != null && BoolCapVal.AUTO.equals(supportsACLs)) {
-            return capabilities.containsKey("ACL");
+        if (BoolCapVal.AUTO == supportsACLs) {
+            Map<String, String> capabilities = this.capabilities;
+            if (capabilities != null) {
+                return capabilities.containsKey("ACL");
+            }
         }
-        return BoolCapVal.TRUE.equals(supportsACLs) ? true : false;
+        return BoolCapVal.TRUE == supportsACLs;
     }
 
     @Override
@@ -343,7 +349,7 @@ public final class IMAPConfig extends MailConfig {
     public InetAddress getImapServerAddress() throws OXException {
         if (null == imapServerAddress) {
             try {
-                imapServerAddress = InetAddress.getByName(imapServer);
+                imapServerAddress = InetAddresses.forString(imapServer);
                 // TODO: Touch address for proper equality check?
                 // imapServerAddress.toString();
             } catch (UnknownHostException e) {
