@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -59,6 +59,7 @@ import com.openexchange.imap.services.Services;
 import com.openexchange.java.Autoboxing;
 import com.openexchange.java.CharsetDetector;
 import com.openexchange.java.Strings;
+import com.openexchange.mail.PreviewMode;
 import com.openexchange.mail.api.AbstractProtocolProperties;
 import com.openexchange.mail.api.IMailProperties;
 import com.openexchange.mail.api.MailConfig.BoolCapVal;
@@ -81,6 +82,8 @@ import net.jodah.failsafe.util.Ratio;
  */
 public final class IMAPProperties extends AbstractProtocolProperties implements IIMAPProperties {
 
+    private static final long DEFAULT_FOLDER_CACHE_TIMEOUT = com.openexchange.imap.cache.ListLsubCache.DEFAULT_TIMEOUT;
+
     private static final org.slf4j.Logger LOG = org.slf4j.LoggerFactory.getLogger(IMAPProperties.class);
 
     private static final IMAPProperties instance = new IMAPProperties();
@@ -112,10 +115,14 @@ public final class IMAPProperties extends AbstractProtocolProperties implements
             int useMultipleAddressesMaxRetryAttempts;
             boolean ignoreDeletedMails;
             boolean includeSharedInboxExplicitly;
+            boolean assumeUserLocalPartForSharedFolderPath;
+            boolean useIMAPEntityDisplayNameIfPossible;
 
             Params() {
                 super();
                 useMultipleAddressesMaxRetryAttempts = -1;
+                useIMAPEntityDisplayNameIfPossible = true;
+                assumeUserLocalPartForSharedFolderPath = false;
             }
         }
 
@@ -135,6 +142,8 @@ public final class IMAPProperties extends AbstractProtocolProperties implements
         final int useMultipleAddressesMaxRetryAttempts;
         final boolean ignoreDeletedMails;
         final boolean includeSharedInboxExplicitly;
+        final boolean assumeUserLocalPartForSharedFolderPath;
+        final boolean useIMAPEntityDisplayNameIfPossible;
 
         PrimaryIMAPProperties(Params params) {
             super();
@@ -152,6 +161,8 @@ public final class IMAPProperties extends AbstractProtocolProperties implements
             this.useMultipleAddressesMaxRetryAttempts = params.useMultipleAddressesMaxRetryAttempts;
             this.ignoreDeletedMails = params.ignoreDeletedMails;
             this.includeSharedInboxExplicitly = params.includeSharedInboxExplicitly;
+            this.assumeUserLocalPartForSharedFolderPath = params.assumeUserLocalPartForSharedFolderPath;
+            this.useIMAPEntityDisplayNameIfPossible = params.useIMAPEntityDisplayNameIfPossible;
         }
     }
 
@@ -347,6 +358,22 @@ public final class IMAPProperties extends AbstractProtocolProperties implements
             args.add(Strings.getLineSeparator());
         }
 
+        {
+            params.assumeUserLocalPartForSharedFolderPath = ConfigViews.getDefinedBoolPropertyFrom("com.openexchange.imap.assumeUserLocalPartForSharedFolderPath", false, view);
+
+            logMessageBuilder.append("  Assume user's local part for a shared folder path: {}{}");
+            args.add(Autoboxing.valueOf(params.assumeUserLocalPartForSharedFolderPath));
+            args.add(Strings.getLineSeparator());
+        }
+
+        {
+            params.useIMAPEntityDisplayNameIfPossible = ConfigViews.getDefinedBoolPropertyFrom("com.openexchange.imap.useIMAPEntityDisplayNameIfPossible", true, view);
+
+            logMessageBuilder.append("  Use IMAP entity's display name (if possible): {}{}");
+            args.add(Autoboxing.valueOf(params.useIMAPEntityDisplayNameIfPossible));
+            args.add(Strings.getLineSeparator());
+        }
+
         PrimaryIMAPProperties primaryIMAPProps = new PrimaryIMAPProperties(params);
         LOG.debug(logMessageBuilder.toString(), args.toArray(new Object[args.size()]));
         return primaryIMAPProps;
@@ -398,12 +425,16 @@ public final class IMAPProperties extends AbstractProtocolProperties implements
 
     private boolean debugLogEnabled;
 
+    private String debugServerPattern;
+
     private boolean overwritePreLoginCapabilities;
 
     private Set<String> propagateHostNames;
 
     private boolean allowFolderCaches;
 
+    private long folderCacheTimeoutMillis;
+
     private boolean allowFetchSingleHeaders;
 
     private String sContainerType;
@@ -418,6 +449,8 @@ public final class IMAPProperties extends AbstractProtocolProperties implements
 
     private List<CommandExecutor> commandExecutors;
 
+    private PreviewMode preferredPreviewMode;
+
     /**
      * Initializes a new {@link IMAPProperties}
      */
@@ -427,6 +460,7 @@ public final class IMAPProperties extends AbstractProtocolProperties implements
         enableTls = true;
         auditLogEnabled = false;
         debugLogEnabled = false;
+        debugServerPattern = null;
         overwritePreLoginCapabilities = false;
         maxNumConnection = -1;
         newACLExtMap = new NonBlockingHashMap<String, Boolean>();
@@ -434,9 +468,11 @@ public final class IMAPProperties extends AbstractProtocolProperties implements
         propagateHostNames = Collections.emptySet();
         allowFetchSingleHeaders = true;
         allowFolderCaches = true;
+        folderCacheTimeoutMillis = DEFAULT_FOLDER_CACHE_TIMEOUT;
         hostExtractingGreetingListener = null;
         enableAttachmentSearch = false;
         commandExecutors = null;
+        preferredPreviewMode = null;
     }
 
     @Override
@@ -458,6 +494,22 @@ public final class IMAPProperties extends AbstractProtocolProperties implements
             args.add(lineSeparator);
         }
 
+        {
+            String str = configuration.getProperty("com.openexchange.imap.folderCacheTimeoutMillis", Long.toString(DEFAULT_FOLDER_CACHE_TIMEOUT)).trim();
+            folderCacheTimeoutMillis = parsePositiveInt(str);
+            if (folderCacheTimeoutMillis < 0) {
+                folderCacheTimeoutMillis = DEFAULT_FOLDER_CACHE_TIMEOUT;
+                logBuilder.append("    Folder Cache Timeout: Invalid value \"{}\". Setting to fallback: {}{}");
+                args.add(str);
+                args.add(Integer.valueOf(imapTimeout));
+                args.add(lineSeparator);
+            } else {
+                logBuilder.append("    Folder Cache Timeout: {}{}");
+                args.add(Long.valueOf(folderCacheTimeoutMillis));
+                args.add(lineSeparator);
+            }
+        }
+
         {
             final String str = configuration.getProperty("com.openexchange.imap.allowFetchSingleHeaders", "true").trim();
             allowFetchSingleHeaders = "true".equalsIgnoreCase(str);
@@ -525,6 +577,16 @@ public final class IMAPProperties extends AbstractProtocolProperties implements
             args.add(lineSeparator);
         }
 
+        {
+            String tmp = configuration.getProperty("com.openexchange.imap.debugLog.serverPattern", "").trim();
+            if (Strings.isNotEmpty(tmp)) {
+                debugServerPattern = tmp;
+                logBuilder.append("    Debug Log Server Pattern: {}{}");
+                args.add(debugServerPattern);
+                args.add(lineSeparator);
+            }
+        }
+
         {
             String tmp = configuration.getProperty("com.openexchange.imap.overwritePreLoginCapabilities", STR_FALSE).trim();
             overwritePreLoginCapabilities = Boolean.parseBoolean(tmp);
@@ -536,7 +598,7 @@ public final class IMAPProperties extends AbstractProtocolProperties implements
         {
             final String tmp = configuration.getProperty("com.openexchange.imap.propagateHostNames", "").trim();
             if (tmp.length() > 0) {
-                propagateHostNames = Collections.unmodifiableSet(new HashSet<String>(Arrays.asList(tmp.split(" *, *"))));
+                propagateHostNames = Collections.unmodifiableSet(new HashSet<String>(Arrays.asList(Strings.splitByComma(tmp))));
             } else {
                 propagateHostNames = Collections.emptySet();
             }
@@ -667,7 +729,7 @@ public final class IMAPProperties extends AbstractProtocolProperties implements
                     args.add(lineSeparator);
                 } else if (tmp.indexOf(':') > 0) {
                     // Expect a comma-separated list
-                    final String[] sa = tmp.split(" *, *");
+                    final String[] sa = Strings.splitByComma(tmp);
                     if (sa.length > 0) {
                         try {
                             final IMAPProtocol imapProtocol = IMAPProtocol.getInstance();
@@ -750,10 +812,13 @@ public final class IMAPProperties extends AbstractProtocolProperties implements
         args.add(spamHandlerName);
         args.add(lineSeparator);
 
-        sslProtocols = configuration.getProperty("com.openexchange.imap.ssl.protocols", "SSLv3 TLSv1").trim();
-        logBuilder.append("    Supported SSL protocols: {}{}");
-        args.add(sslProtocols);
-        args.add(lineSeparator);
+        {
+            final String tmp = configuration.getProperty("com.openexchange.imap.ssl.protocols", "").trim();
+            this.sslProtocols = Strings.isEmpty(tmp) ? null : tmp;
+            logBuilder.append("    Supported SSL protocols: {}{}");
+            args.add(null == this.sslProtocols ? "<default>" : sslProtocols);
+            args.add(lineSeparator);
+        }
 
         {
             final String tmp = configuration.getProperty("com.openexchange.imap.ssl.ciphersuites", "").trim();
@@ -829,6 +894,19 @@ public final class IMAPProperties extends AbstractProtocolProperties implements
             });
         }
 
+        {
+            String tmp = configuration.getProperty("com.openexchange.imap.preferredPreviewMode", "").trim();
+            if (Strings.isNotEmpty(tmp)) {
+                PreviewMode mode = PreviewMode.previewModeFor(tmp);
+                if (mode != null && PreviewMode.NONE != mode) {
+                    preferredPreviewMode = mode;
+                }
+            }
+            logBuilder.append("    Preferred preview mode: {}{}");
+            args.add(Boolean.valueOf(preferredPreviewMode == null ? "<none>" : preferredPreviewMode.getCapabilityName()));
+            args.add(lineSeparator);
+        }
+
         logBuilder.append("Global IMAP properties successfully loaded!");
 
         LOG.info(logBuilder.toString(), args.toArray(new Object[args.size()]));
@@ -1334,6 +1412,7 @@ public final class IMAPProperties extends AbstractProtocolProperties implements
 
     @Override
     protected void resetFields() {
+        CACHE_PRIMARY_PROPS.invalidateAll();
         imapSort = false;
         imapSearch = false;
         forceImapSearch = false;
@@ -1355,10 +1434,11 @@ public final class IMAPProperties extends AbstractProtocolProperties implements
         maxNumConnection = -1;
         sContainerType = "boundary-aware";
         spamHandlerName = null;
-        sslProtocols = "SSLv3 TLSv1";
+        sslProtocols = null;
         cipherSuites = null;
         hostExtractingGreetingListener = null;
         enableAttachmentSearch = false;
+        preferredPreviewMode = null;
         List<CommandExecutor> commandExecutorsList = commandExecutors;
         commandExecutors = null;
         if (null != commandExecutorsList) {
@@ -1514,6 +1594,42 @@ public final class IMAPProperties extends AbstractProtocolProperties implements
         }
     }
 
+    /**
+     * Checks if user's local part should be assumed when determining a shared folder path;<br>
+     * e.g. assume <code>"jane.doe"</code> instead of <code>"jane.doe@invalid.com"</code>.
+     *
+     * @param userId The user identifier
+     * @param contextId The context identifier
+     * @return <code>true</code> to assume user's local part; otherwise <code>false</code>
+     */
+    public boolean assumeUserLocalPartForSharedFolderPath(int userId, int contextId) {
+        try {
+            PrimaryIMAPProperties primaryIMAPProps = getPrimaryIMAPProps(userId, contextId);
+            return primaryIMAPProps.assumeUserLocalPartForSharedFolderPath;
+        } catch (Exception e) {
+            LOG.error("Failed to get assumeUserLocalPartForSharedFolderPath for user {} in context {}. Using default {} instead.", I(userId), I(contextId), Boolean.FALSE, e);
+            return false;
+        }
+    }
+
+    /**
+     * Checks whether shared folders should be displayed using sharing entity's display name or not;<br>
+     * e.g. <code>"Shared folders/Jane Doe"</code> vs. <code>"Shared folders/jane.doe@somewhere.com"</code>.
+     *
+     * @param userId The user identifier
+     * @param contextId The context identifier
+     * @return <code>true</code> to use display name; otherwise <code>false</code>
+     */
+    public boolean useIMAPEntityDisplayNameIfPossible(int userId, int contextId) {
+        try {
+            PrimaryIMAPProperties primaryIMAPProps = getPrimaryIMAPProps(userId, contextId);
+            return primaryIMAPProps.useIMAPEntityDisplayNameIfPossible;
+        } catch (Exception e) {
+            LOG.error("Failed to get useIMAPEntityDisplayNameIfPossible for user {} in context {}. Using default {} instead.", I(userId), I(contextId), Boolean.TRUE, e);
+            return true;
+        }
+    }
+
     /**
      * Checks whether root sub-folders are allowed for primary IMAP server.
      *
@@ -1677,6 +1793,11 @@ public final class IMAPProperties extends AbstractProtocolProperties implements
         return debugLogEnabled;
     }
 
+    @Override
+    public String getDebugServerPattern() {
+        return debugServerPattern;
+    }
+
     @Override
     public boolean isOverwritePreLoginCapabilities() {
         return overwritePreLoginCapabilities;
@@ -1757,6 +1878,11 @@ public final class IMAPProperties extends AbstractProtocolProperties implements
         return mailProperties.hideInlineImages();
     }
 
+    @Override
+    public boolean detectInlineImageByDispositionOnly() {
+        return mailProperties.detectInlineImageByDispositionOnly();
+    }
+
     @Override
     public boolean isAllowNestedDefaultFolderOnAltNamespace() {
         return mailProperties.isAllowNestedDefaultFolderOnAltNamespace();
@@ -1782,6 +1908,11 @@ public final class IMAPProperties extends AbstractProtocolProperties implements
         return allowFolderCaches;
     }
 
+    @Override
+    public long getFolderCacheTimeoutMillis() {
+        return folderCacheTimeoutMillis;
+    }
+
     @Override
     public boolean allowFetchSingleHeaders() {
         return allowFetchSingleHeaders;
@@ -1802,6 +1933,42 @@ public final class IMAPProperties extends AbstractProtocolProperties implements
         return isUserFlagsEnabled() ? enableAttachmentSearch : false;
     }
 
+    @Override
+    public PreviewMode getPreferredPreviewMode() {
+        return preferredPreviewMode;
+    }
+
+    /**
+     * Gets the suitable preview mode for given IMAP configuration.
+     *
+     * @param imapConfig The IMAP configuration
+     * @return The preview mode to use
+     */
+    public static PreviewMode previewModeFor(IMAPConfig imapConfig) {
+        // Get supported capabilities
+        Map<String, String> capabilities = imapConfig.asMap();
+
+        // Check if the preferred preview mode is set & supported
+        PreviewMode preferredPreviewMode = imapConfig.getIMAPProperties().getPreferredPreviewMode();
+        if (preferredPreviewMode != null) {
+            String capabilityName = preferredPreviewMode.getCapabilityName();
+            if (capabilityName != null && capabilities.containsKey(capabilityName)) {
+                // The preferred preview mode is supported
+                return preferredPreviewMode;
+            }
+        }
+
+        // Default behavior for detecting proper preview mode
+        PreviewMode previewMode = PreviewMode.NONE;
+        for (PreviewMode pm : PreviewMode.values()) { // Already iterates is precedence order
+            String capabilityName = pm.getCapabilityName();
+            if (capabilityName != null && capabilities.containsKey(capabilityName)) {
+                previewMode = pm;
+            }
+        }
+        return previewMode;
+    }
+
     private static int parsePositiveInt(String s) {
         try {
             int i = Integer.parseInt(s);
