@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -27,6 +27,8 @@ import java.util.HashSet;
 import java.util.Map;
 import java.util.Set;
 import com.openexchange.java.CharsetDetector;
+import com.openexchange.java.Strings;
+import com.openexchange.mail.PreviewMode;
 import com.openexchange.mail.api.MailConfig.BoolCapVal;
 import com.openexchange.mail.config.MailAccountProperties;
 import com.openexchange.mailaccount.MailAccount;
@@ -84,6 +86,32 @@ public final class MailAccountIMAPProperties extends MailAccountProperties imple
         return lookUpIntProperty("com.openexchange.imap.blockSize", IMAPProperties.getInstance().getBlockSize());
     }
 
+    @Override
+    public PreviewMode getPreferredPreviewMode() {
+        PreviewMode fallbackMode = IMAPProperties.getInstance().getPreferredPreviewMode();
+
+        String modeStr = getAccountProperty("com.openexchange.imap.preferredPreviewMode");
+        if (null != modeStr) {
+            PreviewMode parsedMode = PreviewMode.previewModeFor(modeStr);
+            return parsedMode == null ? fallbackMode : parsedMode;
+        }
+
+        if (mailAccountId == PRIMARY) {
+            modeStr = lookUpProperty("com.openexchange.imap.primary.preferredPreviewMode");
+            if (null != modeStr) {
+                PreviewMode parsedMode = PreviewMode.previewModeFor(modeStr);
+                return parsedMode == null ? fallbackMode : parsedMode;
+            }
+        }
+
+        modeStr = lookUpProperty("com.openexchange.imap.preferredPreviewMode", fallbackMode == null ? "" : fallbackMode.getCapabilityName());
+        if (null != modeStr) {
+            PreviewMode parsedMode = PreviewMode.previewModeFor(modeStr);
+            return parsedMode == null ? fallbackMode : parsedMode;
+        }
+        return fallbackMode;
+    }
+
     @Override
     public int getMaxNumConnection() {
         String tmp = getAccountProperty("com.openexchange.imap.maxNumConnections");
@@ -356,6 +384,23 @@ public final class MailAccountIMAPProperties extends MailAccountProperties imple
         return lookUpBoolProperty("com.openexchange.imap.debugLog.enabled", IMAPProperties.getInstance().isDebugLogEnabled());
     }
 
+    @Override
+    public String getDebugServerPattern() {
+        String tmp = getAccountProperty("com.openexchange.imap.debugLog.serverPattern");
+        if (null != tmp) {
+            return tmp.trim();
+        }
+
+        if (mailAccountId == PRIMARY) {
+            tmp = lookUpProperty("com.openexchange.imap.primary.debugLog.serverPattern");
+            if (null != tmp) {
+                return tmp.trim();
+            }
+        }
+
+        return lookUpProperty("com.openexchange.imap.debugLog.serverPattern", IMAPProperties.getInstance().getDebugServerPattern());
+    }
+
     @Override
     public boolean isOverwritePreLoginCapabilities() {
         String tmp = getAccountProperty("com.openexchange.imap.overwritePreLoginCapabilities");
@@ -377,19 +422,19 @@ public final class MailAccountIMAPProperties extends MailAccountProperties imple
     public Set<String> getPropagateHostNames() {
         String tmp = getAccountProperty("com.openexchange.imap.propagateHostNames");
         if (null != tmp) {
-            return Collections.unmodifiableSet(new HashSet<String>(Arrays.asList(tmp.trim().split(" *, *"))));
+            return Collections.unmodifiableSet(new HashSet<String>(Arrays.asList(Strings.splitByComma(tmp.trim()))));
         }
 
         if (mailAccountId == PRIMARY) {
             tmp = lookUpProperty("com.openexchange.imap.primary.propagateHostNames");
             if (null != tmp) {
-                return Collections.unmodifiableSet(new HashSet<String>(Arrays.asList(tmp.trim().split(" *, *"))));
+                return Collections.unmodifiableSet(new HashSet<String>(Arrays.asList(Strings.splitByComma(tmp.trim()))));
             }
         }
 
         tmp = lookUpProperty("com.openexchange.imap.propagateHostNames");
         if (null != tmp) {
-            return Collections.unmodifiableSet(new HashSet<String>(Arrays.asList(tmp.trim().split(" *, *"))));
+            return Collections.unmodifiableSet(new HashSet<String>(Arrays.asList(Strings.splitByComma(tmp.trim()))));
         }
 
         return IMAPProperties.getInstance().getPropagateHostNames();
@@ -456,6 +501,41 @@ public final class MailAccountIMAPProperties extends MailAccountProperties imple
         return lookUpBoolProperty("com.openexchange.imap.allowFolderCaches", IMAPProperties.getInstance().allowFolderCaches());
     }
 
+    @Override
+    public long getFolderCacheTimeoutMillis() {
+        String folderCacheTimeoutStr = getAccountProperty("com.openexchange.imap.folderCacheTimeoutMillis");
+        if (null != folderCacheTimeoutStr) {
+            try {
+                long timeout = Long.parseLong(folderCacheTimeoutStr.trim());
+                if (timeout <= 0) {
+                    timeout = com.openexchange.imap.cache.ListLsubCache.DEFAULT_TIMEOUT;
+                }
+                return timeout;
+            } catch (NumberFormatException e) {
+                LOG.error("Folder Cache Timeout: Invalid value.", e);
+                return IMAPProperties.getInstance().getFolderCacheTimeoutMillis();
+            }
+        }
+
+        if (mailAccountId == PRIMARY) {
+            folderCacheTimeoutStr = lookUpProperty("com.openexchange.imap.primary.folderCacheTimeoutMillis");
+            if (null != folderCacheTimeoutStr) {
+                try {
+                    long timeout = Long.parseLong(folderCacheTimeoutStr.trim());
+                    if (timeout <= 0) {
+                        timeout = com.openexchange.imap.cache.ListLsubCache.DEFAULT_TIMEOUT;
+                    }
+                    return timeout;
+                } catch (NumberFormatException e) {
+                    LOG.error("Block Size: Invalid value.", e);
+                    return IMAPProperties.getInstance().getFolderCacheTimeoutMillis();
+                }
+            }
+        }
+
+        return lookUpIntProperty("com.openexchange.imap.folderCacheTimeoutMillis", (int) IMAPProperties.getInstance().getFolderCacheTimeoutMillis());
+    }
+
     @Override
     public boolean allowFetchSingleHeaders() {
         String tmp = getAccountProperty("com.openexchange.imap.allowFetchSingleHeaders");
