@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -105,10 +105,14 @@ public class IMAPDebugLoggerGenerator {
             ConfigView view = configViewFactory.getView(userId, contextId);
 
             String filePath = ConfigViews.getDefinedStringPropertyFrom("com.openexchange.imap.debugLog.file.path", "/var/log/open-xchange", view).trim();
-            if (!filePath.endsWith("/")) {
-                filePath = new StringBuilder(filePath).append('/').toString();
+            {
+                StringBuilder filePathBuilder = new StringBuilder(filePath);
+                if (!filePath.endsWith("/")) {
+                    // Ensure ending slash character
+                    filePathBuilder.append('/');
+                }
+                filePath = filePathBuilder.append("imaptrace_").append(contextId).append('_').append(userId).append('_').append(server).toString();
             }
-            filePath = new StringBuilder(filePath).append("imaptrace_").append(contextId).append('_').append(userId).append('_').append(server).toString();
             {
                 File f = new File(filePath);
                 if (f.exists()) {
