@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -22,6 +22,7 @@
 package com.openexchange.imap.entity2acl;
 
 import com.openexchange.exception.OXException;
+import com.openexchange.session.Session;
 
 /**
  * {@link Entity2ACLArgs} - Offers implementation-specific arguments for proper
@@ -41,4 +42,11 @@ public interface Entity2ACLArgs {
 	 */
 	Object[] getArguments(IMAPServer imapServer) throws OXException;
 
+	/**
+	 * Gets the currently applicable session.
+	 *
+	 * @return The session
+	 */
+	Session getSession();
+
 }
