@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -39,7 +39,9 @@ import com.openexchange.imap.cache.RightsCache;
 import com.openexchange.imap.config.IMAPConfig;
 import com.openexchange.imap.ping.IMAPCapabilityAndGreetingCache;
 import com.openexchange.imap.util.HostAndPort;
+import com.openexchange.java.Strings;
 import com.openexchange.mailaccount.MailAccount;
+import com.openexchange.mailaccount.MailAccounts;
 import com.sun.mail.imap.ACL;
 import com.sun.mail.imap.IMAPFolder;
 import com.sun.mail.imap.IMAPStore;
@@ -138,7 +140,7 @@ public final class Entity2ACLAutoDetector {
 
         @Override
         public Entity2ACL call() throws Exception {
-            boolean isPrimary = imapConfig.getAccountId() == MailAccount.DEFAULT_ID;
+            boolean isPrimary = imapConfig.getAccountId() == MailAccount.DEFAULT_ID || MailAccounts.isSecondaryAccount(imapConfig.getAccountId(), imapConfig.getSession());
             String greeting = IMAPCapabilityAndGreetingCache.getGreeting(HostAndPort.instanceFor(serverUrl), imapConfig.isSecure(), imapConfig.getIMAPProperties(), isPrimary);
             return implFor(greeting, imapConfig);
         }
@@ -187,7 +189,7 @@ public final class Entity2ACLAutoDetector {
             /*
              * Return fallback implementation
              */
-            LOG.warn("No IMAP server found that corresponds to greeting:\n\"{}\" on {}.\nSince ACLs are disabled (through IMAP configuration) or not supported by IMAP server, \"{}\" is used as fallback.", info.replaceAll("\r?\n", ""), imapConfig.getServer(), IMAPServer.CYRUS.getName());
+            LOG.warn("No IMAP server found that corresponds to greeting:\n\"{}\" on {}.\nSince ACLs are disabled (through IMAP configuration) or not supported by IMAP server, \"{}\" is used as fallback.", Strings.dropCRLFFrom(info), imapConfig.getServer(), IMAPServer.CYRUS.getName());
             return IMAPServer.CYRUS;
         }
         /*
