@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -26,7 +26,7 @@ package com.openexchange.imap.entity2acl;
  *
  * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
  */
-public class UserGroupID {
+public class UserGroupID implements Comparable<UserGroupID> {
 
     /**
      * The constant representing a failed mapping of a ACL entity name to a system user/group.
@@ -34,8 +34,8 @@ public class UserGroupID {
     public static final UserGroupID NULL = new NullUserGroupID();
 
     private final int id;
-
     private final boolean group;
+    private int hash;
 
     /**
      * Initializes a new {@link UserGroupID}.
@@ -93,12 +93,25 @@ public class UserGroupID {
         return group;
     }
 
+    /**
+     * Checks if identifier denotes a user.
+     *
+     * @return <code>true</code> if identifier denotes a user; otherwise <code>false</code>
+     */
+    public boolean isUser() {
+        return !group;
+    }
+
     @Override
     public int hashCode() {
-        final int prime = 31;
-        int result = 1;
-        result = prime * result + (group ? 1231 : 1237);
-        result = prime * result + id;
+        int result = hash;
+        if (result == 0) {
+            int prime = 31;
+            result = 1;
+            result = prime * result + (group ? 1231 : 1237);
+            result = prime * result + id;
+            hash = result;
+        }
         return result;
     }
 
@@ -123,6 +136,15 @@ public class UserGroupID {
         return true;
     }
 
+    @Override
+    public int compareTo(UserGroupID o) {
+        int c = Boolean.compare(group, o.group);
+        if (c != 0) {
+            return c;
+        }
+        return Integer.compare(id, o.id);
+    }
+
     private static final class NullUserGroupID extends UserGroupID {
 
         public NullUserGroupID() {
