@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -44,9 +44,11 @@ import com.openexchange.config.Reloadable;
 import com.openexchange.config.cascade.ConfigViewFactory;
 import com.openexchange.context.ContextService;
 import com.openexchange.database.DatabaseService;
+import com.openexchange.deputy.DeputyService;
 import com.openexchange.folderstorage.FolderService;
 import com.openexchange.group.GroupService;
 import com.openexchange.groupware.settings.PreferencesItemService;
+import com.openexchange.i18n.TranslatorFactory;
 import com.openexchange.imap.IMAPProvider;
 import com.openexchange.imap.cache.ListLsubCache;
 import com.openexchange.imap.config.IMAPProperties;
@@ -69,6 +71,7 @@ import com.openexchange.mailaccount.MailAccountStorageService;
 import com.openexchange.net.ssl.SSLSocketFactoryProvider;
 import com.openexchange.net.ssl.config.SSLConfigurationService;
 import com.openexchange.net.ssl.config.UserAwareSSLConfigurationService;
+import com.openexchange.notification.service.FullNameBuilderService;
 import com.openexchange.osgi.HousekeepingActivator;
 import com.openexchange.push.PushEventConstants;
 import com.openexchange.secret.osgi.tools.WhiteboardSecretService;
@@ -148,6 +151,9 @@ public final class IMAPActivator extends HousekeepingActivator {
             track(CacheEventService.class, listLsubInvalidator);
             trackService(FolderService.class);
             trackService(AuditLogService.class);
+            trackService(FullNameBuilderService.class);
+            trackService(TranslatorFactory.class);
+            trackService(DeputyService.class);
             openTrackers();
             /*
              * Command provider
@@ -315,8 +321,7 @@ public final class IMAPActivator extends HousekeepingActivator {
                     public void handleEvent(Event event) {
                         int contextId = ((Integer) event.getProperty("com.openexchange.passwordchange.contextId")).intValue();
                         int userId = ((Integer) event.getProperty("com.openexchange.passwordchange.userId")).intValue();
-                        Session session = (Session) event.getProperty("com.openexchange.passwordchange.session");
-                        ListLsubCache.dropFor(session);
+                        ListLsubCache.dropFor(userId, contextId);
                         IMAPStoreCache.getInstance().dropFor(userId, contextId);
                     }
 
