@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -43,6 +43,7 @@ import com.openexchange.imap.util.HostAndPort;
 import com.openexchange.java.Strings;
 import com.openexchange.mail.mime.MimeDefaultSession;
 import com.openexchange.net.ssl.SSLSocketFactoryProvider;
+import com.openexchange.net.ssl.config.SSLConfigurationService;
 import com.sun.mail.iap.ProtocolException;
 import com.sun.mail.imap.GreetingListener;
 import com.sun.mail.imap.protocol.IMAPProtocol;
@@ -360,6 +361,11 @@ public final class IMAPCapabilityAndGreetingCache {
             String sslProtocols = imapProperties.getSSLProtocols();
             if (Strings.isNotEmpty(sslProtocols)) {
                 imapprops.put("mail.imap.ssl.protocols", sslProtocols);
+            } else {
+                SSLConfigurationService sslConfigService = Services.getService(SSLConfigurationService.class);
+                if (sslConfigService != null) {
+                    imapprops.put("mail.imap.ssl.protocols", Strings.toWhitespaceSeparatedList(sslConfigService.getSupportedProtocols()));
+                }
             }
         }
 
@@ -367,6 +373,11 @@ public final class IMAPCapabilityAndGreetingCache {
             String cipherSuites = imapProperties.getSSLCipherSuites();
             if (Strings.isNotEmpty(cipherSuites)) {
                 imapprops.put("mail.imap.ssl.ciphersuites", cipherSuites);
+            } else {
+                SSLConfigurationService sslConfigService = Services.getService(SSLConfigurationService.class);
+                if (sslConfigService != null) {
+                    imapprops.put("mail.imap.ssl.ciphersuites", Strings.toWhitespaceSeparatedList(sslConfigService.getSupportedCipherSuites()));
+                }
             }
         }
     }
