@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -40,7 +40,6 @@ import com.openexchange.session.Session;
 import com.sun.mail.iap.ProtocolException;
 import com.sun.mail.iap.Response;
 import com.sun.mail.imap.IMAPFolder;
-import com.sun.mail.imap.protocol.IMAPProtocol;
 import gnu.trove.list.TIntList;
 import gnu.trove.list.array.TIntArrayList;
 
@@ -75,7 +74,7 @@ public final class IMAPSearch {
 
         MailFields mailFields = new MailFields(MailField.getMailFieldsFromSearchTerm(searchTerm));
         if (mailFields.contains(MailField.BODY) || mailFields.contains(MailField.FULL)) {
-            if (imapConfig.forceImapSearch() || (msgCount >= imapConfig.getIMAPProperties().getMailFetchLimit())) {
+            if (imapConfig.forceImapSearch() || (msgCount > imapConfig.getIMAPProperties().getMailFetchLimit())) {
                 // Too many messages in IMAP folder or IMAP-based search should be forced.
                 // Fall-back to IMAP-based search and accept a non-type-sensitive search.
                 int[] seqNums = issueIMAPSearch(imapFolder, searchTerm, session.getUserId(), session.getContextId());
@@ -273,14 +272,11 @@ public final class IMAPSearch {
             return new int[0];
         }
 
-        final Object oSeqNums = imapFolder.doCommand(new IMAPFolder.ProtocolCommand() {
-            @Override
-            public Object doCommand(IMAPProtocol protocol) throws ProtocolException {
-                try {
-                    return protocol.search(term);
-                } catch (SearchException e) {
-                    throw new ProtocolException(e.getMessage(), e);
-                }
+        final Object oSeqNums = imapFolder.doCommand(protocol -> {
+            try {
+                return protocol.search(term);
+            } catch (SearchException e) {
+                throw new ProtocolException(e.getMessage(), e);
             }
         });
 
