@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -31,6 +31,7 @@ import javax.mail.FolderClosedException;
 import javax.mail.MessagingException;
 import javax.mail.StoreClosedException;
 import javax.mail.search.SearchException;
+import javax.mail.search.SearchTerm;
 import com.openexchange.exception.OXException;
 import com.openexchange.imap.IMAPCapabilities;
 import com.openexchange.imap.IMAPCommandsCollection;
@@ -38,7 +39,7 @@ import com.openexchange.imap.IMAPException;
 import com.openexchange.imap.IMAPException.Code;
 import com.openexchange.imap.config.IMAPConfig;
 import com.openexchange.imap.search.IMAPSearch;
-import com.openexchange.imap.util.WrappingProtocolException;
+import com.openexchange.imap.util.ImapUtility;
 import com.openexchange.java.Strings;
 import com.openexchange.log.LogProperties;
 import com.openexchange.mail.IndexRange;
@@ -50,6 +51,7 @@ import com.sun.mail.iap.BadCommandException;
 import com.sun.mail.iap.CommandFailedException;
 import com.sun.mail.iap.ProtocolException;
 import com.sun.mail.iap.Response;
+import com.sun.mail.iap.WrappingProtocolException;
 import com.sun.mail.imap.IMAPFolder;
 import com.sun.mail.imap.IMAPFolder.ProtocolCommand;
 import com.sun.mail.imap.SortTerm;
@@ -423,12 +425,14 @@ public final class IMAPSort {
             seqNums = sort(sortTerms, jmsSearchTerm, imapFolder, fallbackOnCommandFailed);
         }
 
-        int umlautFilterThreshold = IMAPSearch.umlautFilterThreshold(session.getUserId(), session.getContextId());
-        if (searchTerm != null && umlautFilterThreshold > 0 && seqNums.length <= umlautFilterThreshold && !searchTerm.isAscii()) {
-            /*
-             * Search with respect to umlauts in pre-selected messages
-             */
-            seqNums = IMAPSearch.searchWithUmlautSupport(searchTerm, seqNums, imapFolder);
+        if (searchTerm != null) {
+            int umlautFilterThreshold = IMAPSearch.umlautFilterThreshold(session.getUserId(), session.getContextId());
+            if (umlautFilterThreshold > 0 && seqNums.length <= umlautFilterThreshold && !searchTerm.isAscii()) {
+                /*
+                 * Search with respect to umlauts in pre-selected messages
+                 */
+                seqNums = IMAPSearch.searchWithUmlautSupport(searchTerm, seqNums, imapFolder);
+            }
         }
 
         return new ImapSortResult(seqNums, rangeApplied, sortedByLocalPart);
@@ -450,7 +454,7 @@ public final class IMAPSort {
     public static SortPartialResult sortReturnPartial(SortTerm[] sortTerms, javax.mail.search.SearchTerm jmsSearchTerm, IndexRange indexRange, IMAPFolder imapFolder) throws MessagingException {
         try {
             final String atom = new StringBuilder(16).append(indexRange.start + 1).append(':').append(indexRange.end).toString();
-            return (SortPartialResult) imapFolder.doCommand(new ProtocolCommand() {
+            return (SortPartialResult) ImapUtility.doCommandAndLog(LOG, imapFolder, new ProtocolCommand() {
 
                 @Override
                 public Object doCommand(IMAPProtocol protocol) throws ProtocolException {
@@ -559,30 +563,14 @@ public final class IMAPSort {
         } catch (StoreClosedException e) {
             throw e;
         } catch (MessagingException e) {
-            Exception cause = e.getNextException();
-            if (cause instanceof WrappingProtocolException) {
-                throw ((WrappingProtocolException) cause).getMessagingException();
-            }
             throw e;
         }
     }
 
     private static int[] sort(SortTerm[] sortTerms, javax.mail.search.SearchTerm jmsSearchTerm, IMAPFolder imapFolder, boolean fallbackOnCommandFailed) throws MessagingException, OXException {
         try {
-            return (int[]) imapFolder.doCommand(new ProtocolCommand() {
-
-                @Override
-                public Object doCommand(IMAPProtocol protocol) throws ProtocolException {
-                    try {
-                        return protocol.sort(sortTerms, jmsSearchTerm);
-                    } catch (SearchException e) {
-                        throw new ProtocolException(e.getMessage(), e);
-                    }
-                }
-            });
-        } catch (FolderClosedException e) {
-            throw e;
-        } catch (StoreClosedException e) {
+            return (int[]) ImapUtility.doCommandAndLog(LOG, imapFolder, new SearchAndSortProtocolCommand(sortTerms, jmsSearchTerm));
+        } catch (FolderClosedException | StoreClosedException e) {
             throw e;
         } catch (MessagingException e) {
             if (fallbackOnCommandFailed) {
@@ -640,7 +628,7 @@ public final class IMAPSort {
 
         StringBuilder sb = new StringBuilder();
         for (SortTerm term : terms) {
-            sb.append(term).append(" ");
+            sb.append(term).append(' ');
         }
         sb.setLength(sb.length() - 1);
         return sb.toString();
@@ -691,7 +679,7 @@ public final class IMAPSort {
         /*
          * Cast & return
          */
-        return ((long[]) imapFolder.doCommand(new SORTProtocolCommand(descending, imapFolder)));
+        return ((long[]) ImapUtility.doCommandAndLog(LOG, imapFolder, new SORTProtocolCommand(descending, imapFolder)));
     }
 
     private static final class SORTProtocolCommand implements IMAPFolder.ProtocolCommand {
@@ -699,7 +687,8 @@ public final class IMAPSort {
         private final boolean descending;
         private final IMAPFolder imapFolder;
 
-        public SORTProtocolCommand(boolean descending, IMAPFolder imapFolder) {
+        SORTProtocolCommand(boolean descending, IMAPFolder imapFolder) {
+            super();
             this.descending = descending;
             this.imapFolder = imapFolder;
         }
@@ -754,6 +743,27 @@ public final class IMAPSort {
         }
     } // End of SORTProtocolCommand
 
+    private static final class SearchAndSortProtocolCommand implements IMAPFolder.ProtocolCommand {
+
+        private final SortTerm[] sortTerms;
+        private final SearchTerm jmsSearchTerm;
+
+        SearchAndSortProtocolCommand(SortTerm[] sortTerms, SearchTerm jmsSearchTerm) {
+            super();
+            this.sortTerms = sortTerms;
+            this.jmsSearchTerm = jmsSearchTerm;
+        }
+
+        @Override
+        public Object doCommand(IMAPProtocol protocol) throws ProtocolException {
+            try {
+                return protocol.sort(sortTerms, jmsSearchTerm);
+            } catch (SearchException e) {
+                throw new ProtocolException(e.getMessage(), e);
+            }
+        }
+    } // End of SearchAndSortProtocolCommand
+
     // ---------------------------------------------------------------------------------------------
 
     /**
