@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -26,6 +26,8 @@ import javax.mail.MessagingException;
 import com.openexchange.exception.OXException;
 import com.openexchange.imap.IMAPClientParameters;
 import com.openexchange.java.Streams;
+import com.openexchange.java.Strings;
+import com.openexchange.log.LogProperties;
 import com.openexchange.session.Session;
 import com.sun.mail.imap.IMAPStore;
 
@@ -77,7 +79,8 @@ public abstract class AbstractIMAPStoreContainer implements IMAPStoreContainer {
                 imapStore.setPropagateClientIpAddress(session.getLocalIp());
             }
             try {
-                IMAPClientParameters.setDefaultClientParameters(imapStore, session);
+                String remotePort = LogProperties.get(LogProperties.Name.GRIZZLY_REMOTE_PORT);
+                IMAPClientParameters.setDefaultClientParameters(imapStore, session, Strings.parsePositiveInt(remotePort));
             } catch (OXException e) {
                 throw new MessagingException(e.getMessage(), e);
             }
