@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -22,8 +22,10 @@
 package com.openexchange.imap.storecache;
 
 import java.util.concurrent.TimeUnit;
+import java.util.concurrent.atomic.AtomicReference;
 import java.util.concurrent.locks.LockSupport;
 import javax.mail.MessagingException;
+import com.openexchange.imap.util.IntCounter;
 import com.openexchange.session.Session;
 import com.sun.mail.imap.IMAPStore;
 import com.sun.mail.util.PropUtil;
@@ -36,15 +38,24 @@ import com.sun.mail.util.PropUtil;
  */
 public final class BoundaryAwareIMAPStoreContainer extends UnboundedIMAPStoreContainer {
 
+    private final ThreadLocal<IntCounter> acquired;
     private final int maxRetryCount;
-    private volatile Limiter limiter;
+    private final AtomicReference<Limiter> limiterRef;
 
     /**
      * Initializes a new {@link BoundaryAwareIMAPStoreContainer}.
      */
     public BoundaryAwareIMAPStoreContainer(int accountId, Session session, String server, int port, boolean propagateClientIp, boolean checkConnectivityIfPolled, IMAPStoreCache.Key key) {
         super(accountId, session, server, port, propagateClientIp, checkConnectivityIfPolled, key);
-        maxRetryCount = 10;
+        maxRetryCount = 3;
+        limiterRef = new AtomicReference<>(null);
+        acquired = new ThreadLocal<IntCounter>();
+    }
+
+    @Override
+    public boolean hadAcquired() {
+        IntCounter c = acquired.get();
+        return c != null && c.getCount() > 0;
     }
 
     /**
@@ -53,36 +64,51 @@ public final class BoundaryAwareIMAPStoreContainer extends UnboundedIMAPStoreCon
      * @return The limiter
      */
     private Limiter getLimiter(int max) {
-        Limiter tmp = limiter;
+        Limiter tmp = limiterRef.get();
         if (null == tmp) {
             synchronized (this) {
-                tmp = limiter;
+                tmp = limiterRef.get();
                 if (null == tmp) {
                     tmp = new Limiter(max);
-                    limiter = tmp;
+                    limiterRef.set(tmp);
+                } else {
+                    tmp.setMax(max);
                 }
             }
+        } else {
+            tmp.setMax(max);
         }
         return tmp;
     }
 
     @Override
     public IMAPStore getStore(javax.mail.Session imapSession, String login, String pw, Session session) throws IMAPStoreContainerInvalidException, MessagingException, InterruptedException {
-        final int maxNumAuthenticated = PropUtil.getIntProperty(imapSession.getProperties(), "mail.imap.maxNumAuthenticated", 0);
+        int maxNumAuthenticated = PropUtil.getIntProperty(imapSession.getProperties(), "mail.imap.maxNumAuthenticated", 0);
         if (maxNumAuthenticated <= 0) {
             return super.getStore(imapSession, login, pw, session);
         }
 
-        // Try acquire a permit
-        final Limiter limiter = getLimiter(maxNumAuthenticated);
-        if (limiter.acquire()) {
-            LOG.debug("BoundaryAwareIMAPStoreContainer.getStore(): Acquired -- {}", limiter);
-            return super.getStore(imapSession, login, pw, session);
+        // Check...
+        IntCounter c = acquired.get();
+        if (c != null && c.getCount() > 0) {
+            LOG.debug("BoundaryAwareIMAPStoreContainer.getStore(): Multiple acquisition attempt -- {}", Thread.currentThread().getName());
         }
 
-        // Await until permit is available
+        // Try acquire a permit
+        Limiter limiter = getLimiter(maxNumAuthenticated);
+        if (limiter.acquire()) {
+            return fetchImapStore(imapSession, login, pw, session, c, limiter);
+        }
+
+        // Check thread waiting on itself
+        if (c != null && c.getCount() >= maxNumAuthenticated) {
+            LOG.debug("BoundaryAwareIMAPStoreContainer.getStore(): S E L F   W A I T -- {}", limiter);
+            throw createNoConnectionException(maxNumAuthenticated);
+        }
+
+        // Await possibly available permit
         boolean acquired = false;
-        synchronized (limiter) {
+        if (maxRetryCount > 0) {
             int retryCount = 0;
             do {
                 LOG.debug("BoundaryAwareIMAPStoreContainer.getStore(): W A I T I N G -- {}", limiter);
@@ -90,19 +116,34 @@ public final class BoundaryAwareIMAPStoreContainer extends UnboundedIMAPStoreCon
                 acquired = limiter.acquire();
             } while (retryCount < maxRetryCount && !acquired);
         }
-        
+
         // Check if acquired
         if (acquired) {
-            LOG.debug("BoundaryAwareIMAPStoreContainer.getStore(): Acquired -- {}", limiter);
+            return fetchImapStore(imapSession, login, pw, session, c, limiter);
+        }
+
+        // Timed out -- So what...?
+        LOG.debug("BoundaryAwareIMAPStoreContainer.getStore(): T I M E D   O U T -- {}", limiter);
+        throw createNoConnectionException(maxNumAuthenticated);
+    }
+
+    private IMAPStore fetchImapStore(javax.mail.Session imapSession, String login, String pw, Session session, IntCounter c, Limiter limiter) throws IMAPStoreContainerInvalidException, MessagingException, InterruptedException {
+        LOG.debug("BoundaryAwareIMAPStoreContainer.getStore(): Acquired -- {}", limiter);
+        if (c == null) {
+            // No thread-local counter, yet
+            this.acquired.set(new IntCounter(1));
         } else {
-            // Timed out -- So what...?
-            LOG.debug("BoundaryAwareIMAPStoreContainer.getStore(): T I M E D   O U T -- {}", limiter);
-            // /final String message = "Max. number of connections exceeded. Try again later.";
-            // /throw new MessagingException(message, new com.sun.mail.iap.ConnectQuotaExceededException(message));
+            // There is already a thread-local counter. Just increment it
+            c.increment();
         }
         return super.getStore(imapSession, login, pw, session);
     }
 
+    private static MessagingException createNoConnectionException(int maxNumAuthenticated) {
+        String message = "Max. number of connections (" + maxNumAuthenticated + ") exceeded. Try again later.";
+        return new MessagingException(message, new com.sun.mail.iap.ConnectQuotaExceededException(message));
+    }
+
     /**
      * Performs a wait according to exponential back-off strategy.
      * <pre>
@@ -123,13 +164,17 @@ public final class BoundaryAwareIMAPStoreContainer extends UnboundedIMAPStoreCon
             super.backStore(imapStore);
         } finally {
             // Do not forget to release previously acquired permit
-            final Limiter tmp = limiter;
+            IntCounter c = acquired.get();
+            if (c != null) {
+                c.decrement();
+                if (c.getCount() <= 0) {
+                    this.acquired.remove();
+                }
+            }
+            Limiter tmp = limiterRef.get();
             if (null != tmp) {
                 tmp.release();
-                synchronized (tmp) {
-                    tmp.notifyAll();
-                    LOG.debug("BoundaryAwareIMAPStoreContainer.backStore(): Released -- {}", tmp);
-                }
+                LOG.debug("BoundaryAwareIMAPStoreContainer.backStore(): Released -- {}", tmp);
             }
         }
     }
