@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -394,10 +394,6 @@ public final class IMAPStoreCache {
      * @param session The associated session
      */
     public void returnIMAPStore(IMAPStore imapStore, int accountId, String server, int port, String login, Session session) {
-        if (null == imapStore) {
-            // Nothing to close
-            return;
-        }
         /*
          * Get queue
          */
@@ -409,6 +405,25 @@ public final class IMAPStoreCache {
         container.backStore(imapStore);
     }
 
+    /**
+     * Signals that previously obtained IMAP store instance had been closed.
+     *
+     * @param accountId The account identifier
+     * @param server The host name of the IMAP server
+     * @param port The port
+     * @param login The login/user name
+     * @param session The associated session
+     */
+    public void signalClosed(int accountId, String server, int port, String login, Session session) {
+        IMAPStoreContainer container = map.get(newKey(accountId, server, port, login, session.getUserId(), session.getContextId()));
+        if (null == container) {
+            return;
+        }
+        if (container.hadAcquired()) {
+            container.backStore(null);
+        }
+    }
+
     // -------------------------------------------------------------------------------------------------------------------------------------
 
     private static Key newKey(int accountId, String host, int port, String login, int userId, int contextId) {
