@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -30,18 +30,29 @@ import java.util.concurrent.atomic.AtomicInteger;
  */
 public final class Limiter {
 
-    private final int max;
+    private final AtomicInteger max;
     private final AtomicInteger cur;
 
     /**
      * Initializes a new {@link Limiter}.
+     *
+     * @param max The maximum number of permits
      */
     public Limiter(int max) {
         super();
-        this.max = max;
+        this.max = new AtomicInteger(max);
         cur = new AtomicInteger(max);
     }
 
+    /**
+     * Sets new maximum number of permits.
+     *
+     * @param max The maximum number of permits
+     */
+    public void setMax(int max) {
+        this.max.set(max);
+    }
+
     /**
      * Acquires a permit.
      *
@@ -65,7 +76,7 @@ public final class Limiter {
         int i;
         do {
             i = cur.get();
-            if (i >= max) {
+            if (i >= max.get()) {
                 return;
             }
         } while (!cur.compareAndSet(i, i + 1));
