@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -45,6 +45,11 @@ public class NonCachingIMAPStoreContainer extends AbstractIMAPStoreContainer {
         this.server = server;
     }
 
+    @Override
+    public boolean hadAcquired() {
+        return false;
+    }
+
     @Override
     public IMAPStore getStore(javax.mail.Session imapSession, String login, String pw, Session session) throws MessagingException, InterruptedException {
         return newStore(server, port, login, pw, imapSession, session);
@@ -52,7 +57,9 @@ public class NonCachingIMAPStoreContainer extends AbstractIMAPStoreContainer {
 
     @Override
     public void backStore(IMAPStore imapStore) {
-        backStoreNoValidityCheck(imapStore);
+        if (imapStore != null) {
+            backStoreNoValidityCheck(imapStore);
+        }
     }
 
     protected void backStoreNoValidityCheck(IMAPStore imapStore) {
