@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -72,6 +72,11 @@ public class UnboundedIMAPStoreContainer extends AbstractIMAPStoreContainer {
         this.checkConnectivityIfPolled = checkConnectivityIfPolled;
     }
 
+    @Override
+    public boolean hadAcquired() {
+        return false;
+    }
+
     @Override
     public IMAPStore getStore(javax.mail.Session imapSession, String login, String pw, Session session) throws IMAPStoreContainerInvalidException, MessagingException, InterruptedException {
         // Poll or create IMAP store
@@ -131,6 +136,10 @@ public class UnboundedIMAPStoreContainer extends AbstractIMAPStoreContainer {
 
     @Override
     public void backStore(IMAPStore imapStore) {
+        if (imapStore == null) {
+            return;
+        }
+
         // Try to put back given IMAP store w/o validity check
         boolean enqueued;
         lock.lock();
