@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -53,10 +53,13 @@ import com.openexchange.imap.threader.nntp.ThreadableImpl;
 import com.openexchange.imap.threadsort.ThreadSortNode;
 import com.openexchange.java.Reference;
 import com.openexchange.log.LogProperties;
+import com.openexchange.mail.PreviewMode;
 import com.openexchange.mail.dataobjects.MailMessage;
 import com.openexchange.mail.mime.MessageHeaders;
 import com.openexchange.mail.mime.utils.MimeMessageUtility;
+import com.openexchange.session.IUserAndContext;
 import com.openexchange.session.Session;
+import com.openexchange.session.UserAndContext;
 import com.sun.mail.iap.ProtocolException;
 import com.sun.mail.iap.Response;
 import com.sun.mail.iap.ResponseInterceptor;
@@ -235,12 +238,13 @@ public final class Threadables {
      * @param fetchProfile The FETCH profile
      * @param serverInfo The IMAP server information
      * @param examineHasAttachmentUserFlags Whether has-attachment user flags should be considered
-     * @param previewSupported Whether target IMAP server supports <code>"PREVIEW=FUZZY"</code> capability
+     * @param optUserAndContext The user and context tuple or <code>null</code>
+     * @param previewMode Whether target IMAP server supports any preview capability
      * @return The fetched <tt>MailMessage</tt>s
      * @throws MessagingException If an error occurs
      */
     @SuppressWarnings("unchecked")
-    public static List<MailMessage> getAllMailsFrom(IMAPFolder imapFolder, int limit, FetchProfile fetchProfile, IMAPServerInfo serverInfo, boolean examineHasAttachmentUserFlags, boolean previewSupported) throws MessagingException {
+    public static List<MailMessage> getAllMailsFrom(IMAPFolder imapFolder, int limit, FetchProfile fetchProfile, IMAPServerInfo serverInfo, boolean examineHasAttachmentUserFlags, IUserAndContext optUserAndContext, PreviewMode previewMode) throws MessagingException {
         final int messageCount = imapFolder.getMessageCount();
         if (messageCount <= 0) {
             /*
@@ -270,7 +274,7 @@ public final class Threadables {
                             mails = new ArrayList<MailMessage>(limit);
                         }
                     }
-                    sb.append(" (").append(getFetchCommand(protocol.isREV1(), fetchProfile, false, serverInfo, previewSupported)).append(')');
+                    sb.append(" (").append(getFetchCommand(protocol.isREV1(), fetchProfile, false, serverInfo, previewMode)).append(')');
                     command = sb.toString();
                 }
 
@@ -289,7 +293,7 @@ public final class Threadables {
 
                         if (exceptionRef.hasNoValue()) {
                             try {
-                                MailMessage message = handleFetchRespone((FetchResponse) r, fullName, serverInfo.getAccountId(), examineHasAttachmentUserFlags);
+                                MailMessage message = handleFetchRespone((FetchResponse) r, fullName, serverInfo.getAccountId(), examineHasAttachmentUserFlags, optUserAndContext);
                                 if (null == message.getFirstHeader(sReferences)) {
                                     String inReplyTo = message.getFirstHeader(sInReplyTo);
                                     if (null != inReplyTo) {
