@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -38,16 +38,18 @@ import com.openexchange.imap.IMAPException;
 import com.openexchange.imap.IMAPServerInfo;
 import com.openexchange.imap.command.MailMessageFillerIMAPCommand;
 import com.openexchange.imap.threader.references.Conversation;
-import com.openexchange.imap.util.WrappingProtocolException;
 import com.openexchange.log.LogProperties;
 import com.openexchange.mail.MailField;
+import com.openexchange.mail.PreviewMode;
 import com.openexchange.mail.dataobjects.IDMailMessage;
 import com.openexchange.mail.dataobjects.MailMessage;
 import com.openexchange.mail.mime.utils.MimeStorageUtility;
 import com.openexchange.mail.search.SearchTerm;
+import com.openexchange.session.IUserAndContext;
 import com.sun.mail.iap.Argument;
 import com.sun.mail.iap.ProtocolException;
 import com.sun.mail.iap.Response;
+import com.sun.mail.iap.WrappingProtocolException;
 import com.sun.mail.imap.IMAPFolder;
 import com.sun.mail.imap.protocol.IMAPProtocol;
 import com.sun.mail.imap.protocol.IMAPResponse;
@@ -147,14 +149,15 @@ public final class ThreadSorts {
      * @param sortRange The THREAD command argument specifying the sort range; e.g. <code>&quot;ALL&quot;</code> or <code>&quot;12,13,14,24&quot;</code>
      * @param isRev1 Flag for IMAPrev1
      * @param examineHasAttachmentUserFlags Whether to consider user flags to determine "has attachment(s)" flag
-     * @param previewSupported Whether target IMAP server supports <code>"PREVIEW=FUZZY"</code> capability
+     * @param optUserAndContext The user and context tuple or <code>null</code>
+     * @param previewMode Whether target IMAP server supports any preview capability
      * @param serverInfo The IMAP server information
      * @param fields The optional fields to pre-fill
      * @return The conversation listing
      * @throws OXException If parsing fails
      * @throws MessagingException If acquiring the THREAD response fails
      */
-    public static List<List<MailMessage>> getConversations(IMAPFolder imapFolder, String sortRange, boolean isRev1, boolean examineHasAttachmentUserFlags, boolean previewSupported, IMAPServerInfo serverInfo, SearchTerm<?> searchTerm, MailField... fields) throws OXException, MessagingException {
+    public static List<List<MailMessage>> getConversations(IMAPFolder imapFolder, String sortRange, boolean isRev1, boolean examineHasAttachmentUserFlags, IUserAndContext optUserAndContext, PreviewMode previewMode, IMAPServerInfo serverInfo, SearchTerm<?> searchTerm, MailField... fields) throws OXException, MessagingException {
         if (null == fields || fields.length == 0 || (fields.length == 1 && MailField.RECEIVED_DATE.equals(fields[0]))) {
             return parseConversations(getThreadResponse(imapFolder, sortRange, true, searchTerm), imapFolder.getFullName());
         }
@@ -162,9 +165,9 @@ public final class ThreadSorts {
         // Its fetch profile...
         FetchProfile fp = new FetchProfile();
         for (MailField field : fields) {
-            MimeStorageUtility.addFetchItem(fp, field, examineHasAttachmentUserFlags, previewSupported);
+            MimeStorageUtility.addFetchItem(fp, field, examineHasAttachmentUserFlags, previewMode);
         }
-        return getConversations(imapFolder, sortRange, isRev1, fp, serverInfo, examineHasAttachmentUserFlags, previewSupported, searchTerm);
+        return getConversations(imapFolder, sortRange, isRev1, fp, serverInfo, examineHasAttachmentUserFlags, optUserAndContext, previewMode, searchTerm);
     }
 
     /**
@@ -178,12 +181,13 @@ public final class ThreadSorts {
      * @param fetchProfile The optional fetch profile to pre-fill messages
      * @param serverInfo The IMAP server information
      * @param examineHasAttachmentUserFlags Whether has-attachment user flags should be considered
-     * @param previewSupported Whether target IMAP server supports <code>"PREVIEW=FUZZY"</code> capability
+     * @param previewMode Whether target IMAP server supports any preview capability
+     * @param optUserAndContext The user and context tuple or <code>null</code>
      * @return The conversation listing
      * @throws OXException If parsing fails
      * @throws MessagingException If acquiring the THREAD response fails
      */
-    public static List<List<MailMessage>> getConversations(IMAPFolder imapFolder, String sortRange, boolean isRev1, FetchProfile fetchProfile, IMAPServerInfo serverInfo, boolean examineHasAttachmentUserFlags, boolean previewSupported, SearchTerm<?> searchTerm) throws OXException, MessagingException {
+    public static List<List<MailMessage>> getConversations(IMAPFolder imapFolder, String sortRange, boolean isRev1, FetchProfile fetchProfile, IMAPServerInfo serverInfo, boolean examineHasAttachmentUserFlags, IUserAndContext optUserAndContext, PreviewMode previewMode, SearchTerm<?> searchTerm) throws OXException, MessagingException {
         if (null == fetchProfile) {
             return parseConversations(getThreadResponse(imapFolder, sortRange, true, searchTerm), imapFolder.getFullName());
         }
@@ -199,7 +203,7 @@ public final class ThreadSorts {
                 }
             }
             // Fill them
-            new MailMessageFillerIMAPCommand(msgs, isRev1, fetchProfile, serverInfo, examineHasAttachmentUserFlags, previewSupported, imapFolder).doCommand();
+            new MailMessageFillerIMAPCommand(msgs, isRev1, fetchProfile, serverInfo, examineHasAttachmentUserFlags, optUserAndContext, previewMode, imapFolder).doCommand();
         }
 
         return conversations;
@@ -216,12 +220,13 @@ public final class ThreadSorts {
      * @param fetchProfile The optional fetch profile to pre-fill messages
      * @param serverInfo The IMAP server information
      * @param examineHasAttachmentUserFlags Whether has-attachment user flags should be considered
-     * @param previewSupported Whether target IMAP server supports <code>"PREVIEW=FUZZY"</code> capability
+     * @param previewMode Whether target IMAP server supports any preview capability
+     * @param optUserAndContext The user and context tuple or <code>null</code>
      * @return The conversation listing
      * @throws OXException If parsing fails
      * @throws MessagingException If acquiring the THREAD response fails
      */
-    public static List<Conversation> getConversationList(IMAPFolder imapFolder, String sortRange, boolean isRev1, FetchProfile fetchProfile, IMAPServerInfo serverInfo, boolean examineHasAttachmentUserFlags, boolean previewSupported, SearchTerm<?> searchTerm) throws OXException, MessagingException {
+    public static List<Conversation> getConversationList(IMAPFolder imapFolder, String sortRange, boolean isRev1, FetchProfile fetchProfile, IMAPServerInfo serverInfo, boolean examineHasAttachmentUserFlags, IUserAndContext optUserAndContext, PreviewMode previewMode, SearchTerm<?> searchTerm) throws OXException, MessagingException {
         List<List<MailMessage>> conversations = parseConversations(getThreadResponse(imapFolder, sortRange, true, searchTerm), imapFolder.getFullName());
         {
             // Turn conversations to a flat list
@@ -232,7 +237,7 @@ public final class ThreadSorts {
                 }
             }
             // Fill them
-            new MailMessageFillerIMAPCommand(msgs, isRev1, fetchProfile, serverInfo, examineHasAttachmentUserFlags, previewSupported, imapFolder).doCommand();
+            new MailMessageFillerIMAPCommand(msgs, isRev1, fetchProfile, serverInfo, examineHasAttachmentUserFlags, optUserAndContext, previewMode, imapFolder).doCommand();
         }
 
         List<Conversation> retval = new ArrayList<>(conversations.size());
