@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -38,6 +38,7 @@ import com.openexchange.caching.Cache;
 import com.openexchange.caching.CacheService;
 import com.openexchange.exception.OXException;
 import com.openexchange.folderstorage.Folder;
+import com.openexchange.imap.IMAPCapabilities;
 import com.openexchange.imap.IMAPCommandsCollection;
 import com.openexchange.imap.IMAPFolderStorage;
 import com.openexchange.imap.services.Services;
@@ -226,7 +227,7 @@ public class ExtAccountFolderField implements AdditionalFolderField {
 
                 @Override
                 public Object doCommand(IMAPProtocol protocol) throws ProtocolException {
-                    if (!protocol.hasCapability("METADATA")) {
+                    if (!protocol.hasCapability(IMAPCapabilities.CAP_METADATA)) {
                         // No support for METADATA extension
                         LOG.debug("METADATA not supported by IMAP server {}", protocol.getHost());
                         return null;
