@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -21,15 +21,25 @@
 
 package com.openexchange.imap.util;
 
+import static com.sun.mail.imap.IMAPCommandAndResultHandlers.detachCommandAndResultHandler;
+import static com.sun.mail.imap.IMAPCommandAndResultHandlers.injectCommandAndResultHandler;
 import java.nio.charset.StandardCharsets;
+import java.util.ArrayList;
+import java.util.List;
 import java.util.Locale;
+import java.util.Optional;
 import javax.mail.MessagingException;
 import javax.mail.Store;
+import com.openexchange.java.Strings;
+import com.openexchange.log.LogMessageAndArguments;
 import com.sun.mail.iap.Argument;
+import com.sun.mail.iap.ProtocolException;
 import com.sun.mail.iap.Response;
 import com.sun.mail.imap.IMAPFolder;
+import com.sun.mail.imap.IMAPFolder.ProtocolCommand;
 import com.sun.mail.imap.protocol.BASE64MailboxEncoder;
 import com.sun.mail.imap.protocol.IMAPProtocol;
+import com.sun.mail.imap.protocol.IMAPResponse;
 
 /**
  * {@link ImapUtility} - IMAP utility class.
@@ -198,7 +208,94 @@ public final class ImapUtility {
     private static String surroundWithSingleQuotes(String imapCommand) {
         return new StringBuilder(imapCommand.length() + 2).append('\'').append(imapCommand).append('\'').toString();
     }
-    
+
+   /**
+    * Gets the log arguments for given IMAP command and its associated response.
+    *
+    * @param imapCommand The IMAP command
+    * @param args The associated IMAP command arguments
+    * @param r The response
+    * @param optionalKey The optional key to filter responses by
+    * @return The log arguments
+    */
+   public static LogMessageAndArguments logForImapCommandAndResponse(String imapCommand, Argument args, Response[] r, Optional<String> optionalKey) {
+       StringBuilder logMsg = new StringBuilder();
+       List<Object> logArgs = new ArrayList<>(r.length + 2);
+
+       logMsg.append("{}");
+       logArgs.add(prepareImapCommandForLogging(imapCommand, args));
+
+       for (Response response : r) {
+           if (!(response instanceof IMAPResponse)) {
+               continue;
+           }
+
+           IMAPResponse ir = (IMAPResponse) response;
+           if (!optionalKey.isPresent() || ir.keyEquals(optionalKey.get())) {
+               logMsg.append("{}");
+               logArgs.add(Strings.getLineSeparator());
+
+               logMsg.append("{}");
+               logArgs.add(ir.toString());
+           }
+       }
+
+       return new LogMessageAndArguments(logMsg, logArgs);
+   }
+
+   /**
+    * Execute supplied IMAP command and logs it (with DEBUG log level).
+    *
+    * @param logger The logger to use
+    * @param imapFolder The IMAP folder
+    * @param protocolCommand The protocol command to execute
+    * @return The command's result
+    * @throws MessagingException If commend execution fails
+    */
+   public static Object doCommandAndLog(org.slf4j.Logger logger, IMAPFolder imapFolder, IMAPFolder.ProtocolCommand protocolCommand) throws MessagingException {
+       if (logger.isDebugEnabled()) {
+           return imapFolder.doCommand(new LoggingProtocolCommand(protocolCommand, imapFolder, logger));
+       }
+       return imapFolder.doCommand(protocolCommand);
+   }
+
+   private static final class LoggingProtocolCommand implements IMAPFolder.ProtocolCommand {
+
+       private final ProtocolCommand command;
+       private final com.sun.mail.imap.IMAPCommandAndResultHandler loggingCommandAndResultHandler;
+
+       LoggingProtocolCommand(IMAPFolder.ProtocolCommand command, IMAPFolder imapFolder, org.slf4j.Logger logger) {
+           super();
+           this.command = command;
+           String fullName = imapFolder.getFullName();
+           String imapInfo = imapFolder.getStore().toString();
+           loggingCommandAndResultHandler =  new com.sun.mail.imap.IMAPCommandAndResultHandler() {
+
+               @Override
+               public void handle(String command, Argument args, Response[] responses) throws ProtocolException {
+                   LogMessageAndArguments logArgs = ImapUtility.logForImapCommandAndResponse(command, args, responses, Optional.empty());
+                   logArgs.prependText(" - ");
+                   logArgs.prependArgument(fullName);
+                   if (Strings.isNotEmpty(imapInfo)) {
+                       logArgs.prependText("  ");
+                       logArgs.prependText(imapInfo);
+                   }
+                   logger.debug(logArgs.getLogMessage(), logArgs.getLogArguments());
+               }
+           };
+       }
+
+       @Override
+       public Object doCommand(IMAPProtocol protocol) throws ProtocolException {
+           injectCommandAndResultHandler(loggingCommandAndResultHandler, protocol.getProps());
+           try {
+               return command.doCommand(protocol);
+           } finally {
+               detachCommandAndResultHandler(loggingCommandAndResultHandler, protocol.getProps());
+           }
+       }
+   } // End of class LoggingProtocolCommand
+
     /**
      * Encodes a full folder name. When encoding, capabilities of the protocol are taken into consideration.
      * <p>
